use Test::Simple tests => 17;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::File qw ( GetFileContent DeleteFile );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Cartridge::API::Constants; # STATE_INSTALLED
use DE_EPAGES::Cartridge::API::XML::CartridgesImportHandler qw ( ImportCartridges );
use DE_EPAGES::Cartridge::API::XML::CartridgesDeleteHandler qw ( DeleteCartridges );
use DE_EPAGES::Cartridge::API::Table::Cartridge qw (InsertCartridge GetPKeyCartridgesByPackage GetPKeyCartridgeByPackage DeleteCartridge ExistsCartridgeByPackage);
use DE_EPAGES::Cartridge::API::Table::CartridgeDependency qw (ExistsCartridgeDependency GetInfoCartridgeDependenciesByDependendCartridgeID);
our $CALLED = 0;

sub Main {
    local $| = 1;
    local $ENV{SCRIPT_NAME} = __FILE__;
    RunOnStore( Store => GetTestSite(), Sub => sub {;
        Cleanup();
        ok ( !ExistsCartridgeByPackage($_), "$_ does not yet exist" ) foreach qw(TestCartridge1 TestCartridge2 TestCartridge3 TestCartridge4);

        ImportCartridges('01_s.xml');
        ok ( ExistsCartridgeByPackage($_), "$_ exists" ) foreach qw(TestCartridge1 TestCartridge2 TestCartridge3);
        ok ( !ExistsCartridgeByPackage('TestCartridge4'), "TestCartridge4 does not exist" );

        my $CartidgeID1 = GetPKeyCartridgeByPackage('TestCartridge1');
        my $CartidgeID2 = GetPKeyCartridgeByPackage('TestCartridge2');

        ok( ExistsCartridgeDependency( $CartidgeID1, $CartidgeID2 ), "ExistsCartridgeDependency" );
        ok( !ExistsCartridgeDependency( $CartidgeID2, $CartidgeID1 ), "not ExistsCartridgeDependency" );
        my $aDependencies = GetInfoCartridgeDependenciesByDependendCartridgeID($CartidgeID2);
        ok( 1 == scalar @$aDependencies, "count dependencies" );
        ok( $CartidgeID1 == $aDependencies->[0]->{'CartridgeID'}, "GetInfoCartridgeDependenciesByDependendCartridgeID" );

        ImportCartridges('01_s.xml'); # update

        ok ( Test_Export(),      'correct   export' );
        ok ( Test_ExportFalse(), 'incorrect export' );

        DeleteCartridges('01_s.xml');
        ok ( !ExistsCartridgeByPackage($_), "$_ deleted" ) foreach qw(TestCartridge1 TestCartridge2 TestCartridge3);
        Cleanup();
    });
}

sub Test_Export {
    eval {
        my $Driver = TestDriver->new();
        $Driver->exportXML('01_e.xml',{'encoding' => 'iso-8859-1'});
        my $compareResult = CompareFiles('01_s.xml', '01_e.xml', sub {
            local $_ = shift;
            s/'/"/g;
            s/delete="1"//g;
            s/\s//g;
            return $_;
        });
        Error('NOT_EQUAL') if $compareResult;
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}

sub Test_ExportFalse {
    eval {
        my $HookID = InsertCartridge({
            Package =>'TestCartridge4',
            Version =>'1.4',
            DBStructureVersion =>'1.1',
            InstallState=> STATE_INSTALLED
        });
        my $Driver = TestDriver->new();
        $Driver->exportXML('01_e.xml',{'encoding' => 'iso-8859-1'});
        my $compareResult = CompareFiles('01_s.xml', '01_e.xml', sub {
            local $_ = shift;
            s/'/"/g;
            s/delete="1"//g;
            s/\s//g;
            return $_;
        });
        Error('EQUAL') unless $compareResult;
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}

sub TestHookProc {
    my ($args) = @_;
    Error('NO_HOOK_NAME') unless defined $args->{'HookName'}      and $args->{'HookName'} eq 'TestHook';
    Error('NO_HOOK_PARA') unless defined $args->{'TestParameter'} and $args->{'TestParameter'} eq 'Param';
    $CALLED = 1;
}

sub CompareFiles {
    my ($file1, $file2, $cExec) = @_;
    my $rContent1 = GetFileContent($file1);
    my $rContent2 = GetFileContent($file2);
    if (defined $cExec) {
        $$rContent1 = &$cExec($$rContent1);
        $$rContent2 = &$cExec($$rContent2);
    }
    return $$rContent1 cmp $$rContent2;
}

sub Cleanup {
    DeleteCartridges('01_s.xml');
    DeleteCartridge($_) foreach @{GetPKeyCartridgesByPackage('TestCartridge1')};
    DeleteCartridge($_) foreach @{GetPKeyCartridgesByPackage('TestCartridge2')};
    DeleteCartridge($_) foreach @{GetPKeyCartridgesByPackage('TestCartridge3')};
    DeleteCartridge($_) foreach @{GetPKeyCartridgesByPackage('TestCartridge4')};
    DeleteFile('01_e.xml');
}

RunScript(
    'Sub' => \&Main
);


package TestDriver;
use base qw( DE_EPAGES::Cartridge::API::XML::CartridgesExportDriver );

use strict;
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( GetPKeyCartridgesByPackage );

sub AllCartridges {
    my $self = shift;
    my $IDs = GetPKeyCartridgesByPackage('TestCartridge1');
    $self->addElement( 'Cartridge', { 'CartridgeID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyCartridgesByPackage('TestCartridge2');
    $self->addElement( 'Cartridge', { 'CartridgeID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyCartridgesByPackage('TestCartridge3');
    $self->addElement( 'Cartridge', { 'CartridgeID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyCartridgesByPackage('TestCartridge4');
    $self->addElement( 'Cartridge', { 'CartridgeID' => $_ } ) foreach @$IDs;
}

# get attributes for the document tag
sub epages {
    return {};
}

sub drive {
    my $self = shift;

#    $self->addHook( 'epages' => 'AllCartridges' );
    $self->addCartridgesHooks;

    $self->addElement( 'epages' );
}

1;

