﻿use strict;
use Test::More tests => 42;
use DE_EPAGES::Content::API::ShortUrl qw ( GetObjectByURI GenerateShortUrl );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

use utf8;

my $ROOTPATH = '/Test';
local $ENV{'EPAGES_TESTSITE'} = 'Store' unless defined $ENV{'EPAGES_TESTSITE'};

sub Main {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    local $DE_EPAGES::Object::API::Language::LANGUAGEID = 1;

    my $Shop = LoadObjectByPath('/Test/TestShop/');
    ok( $Shop->alias eq 'TestShop', "Shop loaded" );

    my $RootCategory = $Shop->get('Categories');
    $RootCategory->child('TEST')->delete if $RootCategory->existsChild('TEST');

    #Category without shortUrl
    my $TestCategory = $Shop->get('Categories')->insertChild('Category', { Alias => 'TEST' });
    ok ($TestCategory->get('URI') eq '', 'category without shortUrl');
    ok (defined $TestCategory->get('CanonicalURL'), 'defined CanonicalURL from category without shortUrl');

    #set shortUrl
    $TestCategory->set({'URI' => 'Test'});
    ok ($TestCategory->get('URI') eq 'Test', 'category with shortUrl');
    ok (defined $TestCategory->get('CanonicalURL'), 'defined CanonicalURL from category with shortUrl');

    #set domain name
    my $LocaleID = $Shop->get('LocaleID');
    is ($TestCategory->get('CanonicalURL'), $Shop->get('ProtocolAndServer').'/epages/Store.sf/'.$LocaleID.'/?ObjectPath=/Test/TestShop/Categories/TEST', 'CanonicalURL with objectpath');
    $Shop->set({'DomainName' => 'www.myshop.de'});
    ok ($TestCategory->get('CanonicalURL') eq 'http://www.myshop.de/Test', 'CanonicalURL with shortUrl');

    #find category by url
    my ($ContentObject) = GetObjectByURI($Shop, 'Test');
    ok ($ContentObject->id == $TestCategory->id, 'find category by shortUrl');
    ok (!defined GetObjectByURI($Shop, 'NotExists'), 'cannot find category by unknown shortUrl');

    #find category by upper and lower case
    ($ContentObject) = GetObjectByURI($Shop, 'test');
    ok ($ContentObject->id == $TestCategory->id, 'find category by shortUrl in lower case');
    ($ContentObject) = GetObjectByURI($Shop, 'TEST');
    ok ($ContentObject->id == $TestCategory->id, 'find category by shortUrl in upper case');

    #update shortUrl
    $TestCategory->set({'URI' => 'Test/New'});
    ok ($TestCategory->get('URI') eq 'Test/New', 'update shortUrl');
    ok (!defined GetObjectByURI($Shop, 'test'), 'cannot find category by old shortUrl after update');
    ok (!defined GetObjectByURI($Shop, 'Test'), 'cannot find category by old shortUrl after update');
    ($ContentObject) = GetObjectByURI($Shop, 'test/New');
    ok ($ContentObject->id == $TestCategory->id, 'find category by new shortUrl after update');

    #delete shortUrl (set undef)
    $TestCategory->set({'URI' => undef});
    ok (! defined $TestCategory->get('URI'), 'delete shortUrl by setting undef');
    $TestCategory->set({'URI' => ''});
    ok (! defined $TestCategory->get('URI'), 'delete shortUrl by setting empty');

    #check length
    eval { $TestCategory->set({'URI' => '1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456'}); };
    ok (ExistsError && GetError()->code eq 'STRING_TOO_LONG', 'cannot set url with more than 255 characters');

    eval { $TestCategory->set({'URI' => '123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345'}); };
    ok (!ExistsError, 'can set url with 255 characters');

    #create new category with same shortUrl
    $TestCategory->set({'URI' => 'Test'});
    eval { my $TestCategory2 = $Shop->get('Categories')->insertChild('Category', { Alias => 'TEST2', URI => 'TEST' }); };
    ok (ExistsError && GetError()->code eq 'SHORTURL_ALREADY_EXISTS', 'url already exists');
    eval { my $TestCategory2 = $Shop->get('Categories')->insertChild('Category', { Alias => 'TEST2', URI => 'Test' }); };
    ok (ExistsError && GetError()->code eq 'SHORTURL_ALREADY_EXISTS', 'url already exists');

    #check valid characters
    eval { $TestCategory->set({'URI' => 'Test-6/Test'}) };
    ok (!ExistsError, 'valid characters');
    eval { $TestCategory->set({'URI' => '/Test-6/Test'}) };
    ok (ExistsError && GetError()->code eq 'INVALID_CHARACTER', 'invalid characters: cannot start with "/"');
    eval { $TestCategory->set({'URI' => 'Test_1'}); };
    ok (ExistsError && GetError()->code eq 'INVALID_CHARACTER', 'invalid characters');
    eval { $TestCategory->set({'URI' => 'Test 1'}); };
    ok (ExistsError && GetError()->code eq 'INVALID_CHARACTER', 'invalid characters');
    eval { $TestCategory->set({'URI' => 'Test%201'}); };
    ok (ExistsError && GetError()->code eq 'INVALID_CHARACTER', 'invalid characters');

    #cannot set shortUrl for startpage
    eval { $RootCategory->set({'URI' => 'Start'}); };
    ok (ExistsError && GetError()->code eq 'ShortURL_NOT_ALLOWED', 'cannot set shortUrl for startpage');

    #generate short url
    $TestCategory->set({'URI' => 'Test'});
    ok (GenerateShortUrl($Shop, 'Test123') eq 'Test123', 'generate shortUrl Test123');
    ok (GenerateShortUrl($Shop, 'Test 123_456') eq 'Test-123456', 'generate shortUrl Test-123456');
    ok (GenerateShortUrl($Shop, 'Hündchenölgemälde mit Straße') eq 'Huendchenoelgemaelde-mit-Strasse', 'generate shortUrl Huendchenoelgemaelde-mit-Strasse');
    ok (GenerateShortUrl($Shop, ' test @ test') eq 'test-test', 'generate shortUrl test-test');
    ok (GenerateShortUrl($Shop, 'Ποσειδων') eq 'Poseidon', 'generate shortUrl Poseidon');
    ok (GenerateShortUrl($Shop, '  ∫√ﬁ∆ℓ') eq '', 'generate shortUrl (empty)');
    ok (GenerateShortUrl($Shop, 'Test') eq 'Test-1', 'generate shortUrl with existing url');
    ok (GenerateShortUrl($Shop, '/Test/1/') eq 'Test/1', 'generate shortUrl with /');

    #with temporary list
    my $ahTemporary = [{'Uri' => 'Test1'}, {'Uri' => 'Test/en'}];
    ok (GenerateShortUrl($Shop, 'Test1', 1, $ahTemporary) eq 'Test1/de', 'generate shortUrl with existing url in temporary list');
    ok (GenerateShortUrl($Shop, 'Test', 2, $ahTemporary) eq 'Test-1/en', 'generate shortUrl with existing url in temporary list');

    #generate short url with 250 characters (+ counter)
    ok (
        GenerateShortUrl(
            $Shop,
            'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.'
        ) eq 'Lorem-ipsum-dolor-sit-amet-consetetur-sadipscing-elitr-sed-diam-nonumy-eirmod-tempor-invidunt-ut-labore-et-dolore-magna-aliquyam-erat-sed-diam-voluptua-At-vero-eos-et-accusam-et-justo-duo-dolores-et-ea-rebum-Stet-clita-kasd-gubergren-no-sea-takimata', 'generate shortUrl from long name'
    );
    $TestCategory->set({'URI' => 'Lorem-ipsum-dolor-sit-amet-consetetur-sadipscing-elitr-sed-diam-nonumy-eirmod-tempor-invidunt-ut-labore-et-dolore-magna-aliquyam-erat-sed-diam-voluptua-At-vero-eos-et-accusam-et-justo-duo-dolores-et-ea-rebum-Stet-clita-kasd-gubergren-no-sea-takimata'});
    ok (
        GenerateShortUrl(
            $Shop,
            'Lorem ipsum dolor sit amet, consetetur sadipscing elitr, sed diam nonumy eirmod tempor invidunt ut labore et dolore magna aliquyam erat, sed diam voluptua. At vero eos et accusam et justo duo dolores et ea rebum. Stet clita kasd gubergren, no sea takimata sanctus est Lorem ipsum dolor sit amet.'
        ) eq 'Lorem-ipsum-dolor-sit-amet-consetetur-sadipscing-elitr-sed-diam-nonumy-eirmod-tempor-invidunt-ut-labore-et-dolore-magna-aliquyam-erat-sed-diam-voluptua-At-vero-eos-et-accusam-et-justo-duo-dolores-et-ea-rebum-Stet-clita-kasd-gubergren-no-sea-takimata-1', 'generate shortUrl from long name'
    );
    ok (
        GenerateShortUrl(
            $Shop,
            'Lorem/ipsum/dolor sit amet/consetetur/sadipscing elitr'
        ) eq 'Lorem/ipsum/dolor-sit-amet/consetetur/sadipscing-elitr', 'generate shortUrl with path'
    );

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
