use strict;
use Test::More;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::File qw( ExistsDirectory ExistsFile DeleteDirectory DeleteFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath);
use DE_EPAGES::WebInterface::API::FakeRequest;

#========================================================================================
# function     TimeThis
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Time = TimeThis( $Code );
#----------------------------------------------------------------------------------------
# description  Executes a code block and returns the execution time.
#----------------------------------------------------------------------------------------
# input        $Code | code to evaluate | code reference
# return       $Time | execution time in seconds | float
#========================================================================================
sub TimeThis {
    my ($Code) = @_;

    my $time0 = time();
    $Code->();
    return time() - $time0;
}

#========================================================================================
# function     Main
# state        private
#========================================================================================
sub Main {

    my $NumTests = 3;
    plan tests => $NumTests;

    my $ObjectPath = "/Shops/DemoShop";
    my ($AdminURL, $StyleID, $ShopID, $PrivatePath);
    RunOnStore(
        Store      => GetTestSite(),
        Sub        => sub {
            if( ExistsObjectByPath( $ObjectPath ) ) {
                my $Shop = LoadObjectByPath( $ObjectPath );
                $AdminURL = $Shop->get('WebUrlAdminSSL');
                $StyleID = $Shop->get('DefaultStyle')->id;
                $ShopID = $Shop->id;
                $PrivatePath = $Shop->get('PrivatePath');
            }
        }
    );

    SKIP: {
        skip "no DemoShop installed", $NumTests unless defined $AdminURL;

        my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
        my $GetURL = sub {
            my ($URL) = @_;

            my $Response = $FakeRequest->get( $URL );
            if( !$Response->is_success ) {
                die $Response->status_line;
            }
            else {
                return $Response->content;
            }
        };

        # Get this URL without timing first call.
        my $URL = $AdminURL.'?ViewAction=MBO-ViewStyles&ObjectPath='.$ObjectPath.'&ChangeAction=SaveLoginForm&Login=admin&Password=admin&ChangeObjectID='.$ShopID.'&ChangeAction=ExportStyles&ChangeObjectID='.$ShopID.'&ListObjectID='.$StyleID;
        $GetURL->($URL);

        my $time = TimeThis( sub {
            my $URL2 = _url($AdminURL, 'ViewAction=MBO-ViewStyles&ObjectPath='.$ObjectPath.'&ChangeAction=SaveLoginForm&Login=admin&Password=admin&ChangeObjectID='.$ShopID.'&ChangeAction=ExportStyles&ChangeObjectID='.$ShopID.'&ListObjectID='.$StyleID);
            $GetURL->($URL2);
        });
        cmp_ok($time, '<', 4, "time for export to zip");

        $time = TimeThis( sub {

            my $URL3 = _url($AdminURL, 'ViewAction=ViewStyleExportResult&ObjectPath='.$ObjectPath.'&ChangeAction=SaveLoginForm&Login=admin&Password=admin');
            cmp_ok(length($GetURL->($URL3)), '>', 10000, "file is not empty: ".length($GetURL->($URL3))." bytes");
        });
        cmp_ok($time, '<', 1, "time for reading zip");

        DeleteDirectory($PrivatePath . '/ExportStyle') if ExistsDirectory($PrivatePath . '/ExportStyle');
        DeleteFile($PrivatePath . '/export.zip') if ExistsFile($PrivatePath . '/export.zip');
    };

    return;
}

sub _url {
    my ($URL, $ParameterString) = @_;
    if ($URL =~ /\/\?/) {
        $URL .= '&'.$ParameterString;
    } else {
        $URL .= '/?'.$ParameterString;
    }
}

RunScript( Sub => \&Main, Debug => 1 );
