use strict;
use Test::Simple tests => 377;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::TestParameter qw( ExtractAddrSpec TestEMailAddress TestMultipleEMailAddresses );

sub Main {

    #--------------------------------------------------------------------
    # TestEMailAddress (UTF8/RFC2822)
    #--------------------------------------------------------------------

    use utf8;
    my %Utf8Tests = (
        '!#$%&\'*+-/=?^_`{|}~@epages.de' => '',
        '"Günther" <ba@günther.de>'    => '',
        '"§()[]@epages.de'              => 'MAIL_ADDRESS_PARSE_ARRAY_GT_1',
        '.@epages.de'                    => 'ADDR_LOCAL_ATOM_EMPTY',
        'Doe <.doe@example.com>'         => 'ADDR_LOCAL_ATOM_EMPTY',
        'Doe <doé.@example.com>'        => 'ADDR_LOCAL_ATOM_EMPTY',
        'ba@günter.de'                  => '',
        'blechschmidt@212.1.25.6'        => '',
        'double..dot@epages.de'          => 'ADDR_LOCAL_ATOM_EMPTY',
        'max <test@example.com'          => 'MAIL_ADDRESS_PARSE_FAILED',
        'max <test@example.com>,'        => '',
        'max'                            => 'DOMAIN_NAME_EMPTY',
        'max.mustermann@epages.de'       => '',
        'max.mustermann\@e+pages.de'     => 'MAIL_ADDRESS_PARSE_ARRAY_GT_1',
        'noat.epages.de'                 => 'DOMAIN_NAME_EMPTY',
        'äöü@epages.de'               => '',
        'éééè@epages.de'             => '',
        );
    no utf8;

    _test_email_address(\%Utf8Tests);

    #--------------------------------------------------------------------
    # ExtractAddrSpec (RFC2822/5322)
    #--------------------------------------------------------------------

    my %MsgFmtTests = (
        2822 => {
            "   " => "",
            "aaa" => "aaa",
            "(b<xx>bb) aa1 <w f\@<de.><d>e> <.x\n\nxx> yyy" => "wf\@de.de.xxx",
            "(b<xx>bb) aa2, <wf\@de.de> yyy" => "MAIL_ADDRESS_PARSE_ARRAY_GT_1",
            "(b<xx>bb) aa3 <w,f\@de.de> yyy" => "MAIL_ADDRESS_PARSE_FAILED",
            "(b<x,x>bb) aa4 <wf\@de.de> yyy" => "wf\@de.de",
            "\"b<x,x>bb\" aa5 <wf\@de.de> yyy" => "wf\@de.de",
            "\"b<xx>bb\" aa6 <w f\@<de.<d>e<.xxx yyy>" => "MAIL_ADDRESS_PARSE_FAILED",
            "\"b<xx>bb\" aa7 <w f\@<de.<de<.xxx yyy" => "MAIL_ADDRESS_PARSE_FAILED",
            "\"b<xx>bb\" ( aa8 <w f\@<de.<d>e<.xxx> yyy" => "MAIL_ADDRESS_PARSE_FAILED",
            "\"b<xx>bb aa9 <wf\@de.de> yyy" => "xxwf\@de.de",
            "aaa < ,, \@ep.de ,\@jade,\@gx.de:wf\@de.de> yyy" => "MAIL_ADDRESS_PARSE_FAILED",
            "aaa <\@ep-.de:wf\@de.de> yyy" => "\@ep-.de:wf\@de.de",
            "aaa <\@ep.de ,\@ja.de,wf\@de.de> yyy" => "MAIL_ADDRESS_PARSE_FAILED",
        },
        5322 => {
            "   " => "",
            "aaa" => "aaa",
            "(b<xx>bb) aa1 <w f\@<de.><d>e> <.x\n\nxx> yyy" => "w f\@<de.",
            "(b<xx>bb) aa2, <wf\@de.de> yyy" => "wf\@de.de",
            "(b<xx>bb) aa3 <w,f\@de.de> yyy" => "w,f\@de.de",
            "(b<x,x>bb) aa4 <wf\@de.de> yyy" => "wf\@de.de",
            "\"b<x,x>bb\" aa5 <wf\@de.de> yyy" => "wf\@de.de",
            "\"b<xx>bb\" aa6 <w f\@<de.<d>e<.xxx yyy>" => "w f\@<de.<d",
            "\"b<xx>bb\" aa7 <w f\@<de.<de<.xxx yyy" => "ADDR_UNMATCHED_ANGLE_BRACKETS",
            "\"b<xx>bb\" ( aa8 <w f\@<de.<d>e<.xxx> yyy" => "ADDR_UNMATCHED_PARENTHESIS",
            "\"b<xx>bb aa9 <wf\@de.de> yyy" => "ADDR_UNMATCHED_QUOTE",
            "aaa < ,, \@ep.de ,\@ja.de,\@gx.de:wf\@de.de> yyy" => "wf\@de.de",
            "aaa <\@ep-.de:wf\@de.de> yyy" => "DOMAIN_BOUNDARY_HYPHEN",
            "aaa <\@ep.de ,\@ja.de,wf\@de.de> yyy" => "\@ep.de ,\@ja.de,wf\@de.de",
        }
        );

    _extract_addr_spec($MsgFmtTests{$_}, $_) foreach qw(2822 5322);

    #--------------------------------------------------------------------
    # TestEMailAddress (RFC2821/5321)
    #--------------------------------------------------------------------

    my %SmtpTests = (

        common => {
            # length
            "+\@b.c" => "",
            "12345678901234567890123456789012345678901234567890123456789012345\@example.com" => "ADDR_LOCAL_GT_64_CHARS",
            "1234567890123456789012345678901234567890123456789012345678901234\@example.com" => "",
            "123456789012345678901234567890123456789012345678901234567890\@12345678901234567890123456789012345678901234567890123456789.12345678901234567890123456789012345678901234567890123456789.12345678901234567890123456789012345678901234567890123456789.1234.example.com" => "ADDR_STRING_GT_256_CHARS",
            "123456789012345678901234567890123456789012345678901234567890\@12345678901234567890123456789012345678901234567890123456789.12345678901234567890123456789012345678901234567890123456789.123456789012345678901234567890123456789012345678901234567890123.example.com" => "",
            "first.last\@(1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890)example.com" => "ADDR_STRING_GT_256_CHARS",
            "first.last\@x(1234567890123456789012345678901234567890123456789012345678901234567890).com" => "",
            "first.last\@x23456789012345678901234567890123456789012345678901234567890123.example.com" => "",
            "first.last\@x234567890123456789012345678901234567890123456789012345678901234.example.com" => "DOMAIN_ATOM_GT_63_CHARS",
            "x\@x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x234" => "DOMAIN_NAME_GT_253_CHARS",
            "x\@x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456789.x23456" => "ADDR_STRING_GT_256_CHARS",

            # special characters
            "aaa\@[123.123.123.123]" => "",
            "!def!xyz%abc\@example.com" => "",
            "(\@example.com" => "ADDR_UNMATCHED_PARENTHESIS",
            ")\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "><\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            ";\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            ":\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "(<)x\@example.com" => "",
            "(,)x\@example.com" => "",
            "(\\\")x\@example.com" => "",
            "(\\\))x\@example.com" => "",
            "\"x\\\"y\"\@example.com" => "",
            "1234567890\@example.com" => "",
            "Ima Fool\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "[test]\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "\$A12345\@example.com" => "",
            "_Yosemite.Sam\@example.com" => "",
            "customer/department=shipping\@example.com" => "",
            "customer/department\@example.com" => "",
            "t*est\@example.com" => "",
            "test+test\@example.com" => "",
            "test-test\@example.com" => "",
            "user%uucp!path\@somehost.edu" => "",
            "user+mailbox\@example.com" => "",
            "{^c\\\@**Dog^}\@cartoon.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "{_test_}\@example.com" => "",
            "~\@example.com" => "",

            # escaped characters
            "Joe.\\\\Blow\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "Fred\\ Bloggs\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "Abc\\\@def\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "Doug\\ \\\"Ace\\\"\\ L\\.\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "Doug\\ \\\"Ace\\\"\\ Lovell\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "Invalid \\\n Folding \\\n Whitespace\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "abc\\\@def\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "abc\\\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "abc\\\\\@def\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "abc\\\\\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "phil.h\\\@\\\@ck\@haacked.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",

            # (folding) whitespaces
            " a\@b" => "",
            "\r\n \ta\@b" => "",
            " a \@ b " => "",
            " a\@b " => "",
            "\" a\t\\\rb \"\@c" => "",
            "\" a\t\\\rb \"\b\@c" => "ADDR_UNMATCHED_QUOTE",
            "\" a\tb \"\@\"c\t d\"" => "DOMAIN_CHARS_VOID",
            "-- test --\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "1234   \@   local(blah)  .machine .example" => "",
            "Test.\r\n Folding.\r\n Whitespace\@example.com" => "",
            "\"test\\\r\n blah\"\@example.com" => "",
            "\"test\\\rblah\"\@example.com" => "",
            "\"test\\\nblah\"\@example.com" => "ADDR_NO_SPACE_AFTER_NEWLINE",
            "\"test\\\"\@example.com" => "ADDR_UNMATCHED_QUOTE",
            "\"test\r\nxx\"\@example.com" => "ADDR_NO_SPACE_AFTER_NEWLINE",
            "\"test\rblah\"\@example.com" => "ADDR_UNQUOTED_WHITESPACE",
            "\"test\x00blah\"\@example.com" => "ADDR_UNQUOTED_NUL",
            "\"test\r\n blah\"\@example.com" => "",
            "hello world\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "jdoe\@machine(comment).  example" => "",

            # dot atoms
            ".\@" => "ADDR_LOCAL_ATOM_EMPTY",
            "-\@..com" => "DOMAIN_ATOM_EMPTY",
            "-\@a..com" => "DOMAIN_ATOM_EMPTY",
            ".\@example.com" => "ADDR_LOCAL_ATOM_EMPTY",
            ".dot\@example.com" => "ADDR_LOCAL_ATOM_EMPTY",
            ".first.last\@example.com" => "ADDR_LOCAL_ATOM_EMPTY",
            "Ima.Fool\@example.com" => "",
            "NotAnEmail" => "DOMAIN_NAME_EMPTY",
            "TEST\@example.com" => "",
            "\@NotAnEmail" => "ADDR_LOCAL_EMPTY",
            "\@\@bar.com" => "ADDR_TOO_MUCH_AT_SIGNS",
            "\@bar.com" => "ADDR_LOCAL_EMPTY",
            "a\@b" => "",
            "a\@bar.com." => "DOMAIN_ATOM_EMPTY",
            "aaa.com" => "DOMAIN_NAME_EMPTY",
            "aaa\@.123" => "DOMAIN_ATOM_EMPTY",
            "abc\@def\@example.com" => "ADDR_TOO_MUCH_AT_SIGNS",
            "dclo\@us.ibm.com" => "",
            "dot.\@example.com" => "ADDR_LOCAL_ATOM_EMPTY",
            "doug\@" => "DOMAIN_NAME_EMPTY",
            "first..last\@example.com" => "ADDR_LOCAL_ATOM_EMPTY",
            "first.\"\".last\@example.com" => "ADDR_LOCAL_ATOM_EMPTY",
            "first.\"last\"\@example.com" => "",
            "first.last.\@example.com" => "ADDR_LOCAL_ATOM_EMPTY",
            "first.last\@" => "DOMAIN_NAME_EMPTY",
            "gatsby\@f.sc.ot.t.f.i.tzg.era.l.d." => "DOMAIN_ATOM_EMPTY",
            "test\@." => "DOMAIN_ATOM_EMPTY",
            "test\@...........com" => "DOMAIN_ATOM_EMPTY",
            "test\@.org" => "DOMAIN_ATOM_EMPTY",
            "test\@\@example.com" => "ADDR_TOO_MUCH_AT_SIGNS",
            "test\@example.example.example.com" => "",
            "two..dot\@example.com" => "ADDR_LOCAL_ATOM_EMPTY",

            # domain name
            "a\@-b.com" => "DOMAIN_BOUNDARY_HYPHEN",
            "a\@b-.com" => "DOMAIN_BOUNDARY_HYPHEN",
            "a\@b.co-foo.uk" => "",
            "first.last\@123.example.com" => "",
            "first.last\@1xample.com" => "",
            "first.last\@com" => "",
            "first.last\@exampl-.com" => "DOMAIN_BOUNDARY_HYPHEN",
            "first.last\@example.123" => "DOMAIN_TLD_ALL_NUMBERS",
            "test\@123.123.123.x123" => "",
            "test\@example" => "",
        },

        2821 => {

            # special characters
            "aaa\@[123.123.123.123]" => "ADDR_NET_SMTP_VOID_BRACKET",
            "(,)x\@example.com" => "ADDR_NET_SMTP_VOID_PARENTHESIS",
            "(<)x\@example.com" => "ADDR_NET_SMTP_VOID_PARENTHESIS",
            "(\\\")x\@example.com" => "ADDR_NET_SMTP_VOID_PARENTHESIS",
            "(\\\))x\@example.com" => "ADDR_NET_SMTP_VOID_PARENTHESIS",
            "first.last\@x(1234567890123456789012345678901234567890123456789012345678901234567890).com" => "ADDR_NET_SMTP_VOID_PARENTHESIS",

            # (folding) whitespaces
            " a \@ b " => "ADDR_NET_SMTP_INNER_SPACES",
            "a .s\@b" => "ADDR_NET_SMTP_INNER_SPACES",
            " a\@b " => "",
            "-- test --\@example.com" => "ADDR_NET_SMTP_INNER_SPACES",
            "1234   \@   local(blah)  .machine .example" => "ADDR_NET_SMTP_INNER_SPACES",
            "Ima Fool\@example.com" => "ADDR_NET_SMTP_INNER_SPACES",
            "Test.\r\n Folding.\r\n Whitespace\@example.com" => "ADDR_NET_SMTP_VOID_NEWLINE",
            "\" a\t\\\rb \"\@c" => "ADDR_NET_SMTP_VOID_NEWLINE",
            "\" a\tb \"\@\"c\t d\"" => "ADDR_NET_SMTP_INNER_SPACES",
            "\"test\\\nblah\"\@example.com" => "ADDR_NET_SMTP_VOID_NEWLINE",
            "\"test\\\r\n blah\"\@example.com" => "ADDR_NET_SMTP_VOID_NEWLINE",
            "\"test\\\rblah\"\@example.com" => "ADDR_NET_SMTP_VOID_NEWLINE",
            "\"test\r\n blah\"\@example.com" => "ADDR_NET_SMTP_VOID_NEWLINE",
            "\"test\r\nxx\"\@example.com" => "ADDR_NET_SMTP_VOID_NEWLINE",
            "\"test\rblah\"\@example.com" => "ADDR_NET_SMTP_VOID_NEWLINE",
            "\r\n \ta\@b" => "ADDR_NET_SMTP_VOID_NEWLINE",
            "hello world\@example.com" => "ADDR_NET_SMTP_INNER_SPACES",
            "jdoe\@machine(comment).  example" => "ADDR_NET_SMTP_VOID_PARENTHESIS",

            # quotes
            "\"a,b\"\@epages.de" => "",
            "\"a\\,b\"\@epages.de" => "",
            "\"Abc\@def\"\@example.com" => "",
            "\"Abc\\\@def\"\@example.com" => "",
            "\"Doug \"Ace\" L.\"\@example.com" => "ADDR_NET_SMTP_INNER_QUOTES",
            "\"Doug \\\"Ace\\\" L.\"\@example.com" => "",
            "\"Doug \\\"\\(Ace\\)\\\" L.\"\@example.com" => "",
            "\"first\".\"middle\".\"last\"\@example.com" => "ADDR_NET_SMTP_INNER_QUOTES",
        },

        5321 => {

            # comments
            " \r\n (\r\n x \r\n ) \r\n first\r\n ( \r\n x\r\n ) \r\n .\r\n ( \r\n x) \r\n last \r\n (  x \r\n ) \r\n \@example.com" => "",
            "+\@b.c (asde)" => "",
            "(asde) +\@b.c" => "",
            "(foo)(bar)cal(bar)\@(baz)iamcal.com(quux)" => "",
            "HM2Kinsists\@(that comments are allowed)this.is.ok" => "",
            "\"foo\"(yay)\@(hoopla)[1.2.3.4]" => "ADDR_WITH_IP_COMMENT",
            "a(a(b(c)d(e(f))g)(h(i)j)\@example.com" => "ADDR_UNMATCHED_PARENTHESIS",
            "a(a(b(c)d(e(f))g)h(i)j)\@example.com" => "",
            "c\@(Chris\'s host.)public.example" => "",
            "cal(foo(bar)\@iamcal.com" => "ADDR_UNMATCHED_PARENTHESIS",
            "cal(foo)bar)\@iamcal.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "cal(foo\\)\@iamcal.com" => "ADDR_UNMATCHED_PARENTHESIS",
            "cal(foo\\)bar)\@iamcal.com" => "",
            "cal(foo\\\@bar)\@iamcal.com" => "",
            "cal(woo(yay)hoopla)\@iamcal.com" => "",
            "cal\@iamcal(woo).(yay)com" => "",
            "first().last\@example.com" => "",
            "first(Welcome to\r\n the (\"wonderful\" (!)) world\r\n of email)\@example.com" => "",
            "first(a\"bc.def).last\@example.com" => "",
            "first(abc(\"def\".ghi).mno)middle(abc(\"def\".ghi).mno).last\@(abc(\"def\".ghi).mno)example(abc(\"def\".ghi).mno).(abc(\"def\".ghi).mno)com(abc(\"def\".ghi).mno)" => "ADDR_COMMENT_NOT_ON_BOUNDARY",
            "first(abc.def).last\@example.com" => "",
            "first(abc\\(def)\@example.com" => "",
            "first(middle)last\@example.com" => "ADDR_COMMENT_NOT_ON_BOUNDARY",
            "first.(\")middle.last(\")\@example.com" => "",
            "first.(\r\n middle\r\n )last\@example.com" => "",
            "pete(his account)\@silly.test(his host)" => "",

            # IP address
            "aaa\@[123.123.123.123]a" => "DOMAIN_CHARS_VOID",
            "aaa\@[123.123.123.333]" => "VOID_IPv4_ADDRESS",
            "first.last\@[.12.34.56.78]" => "VOID_IPv4_ADDRESS",
            "first.last\@[12.34.56.789]" => "VOID_IPv4_ADDRESS",
            "first.last\@[12.34.56.78]" => "",
            "first.last\@[::12.34.56.78]" => "ADDR_IPv6_TAG_MISSING",
            "first.last\@[IPv5:::12.34.56.78]" => "ADDR_IPv6_TAG_MISSING",
            "first.last\@[IPv6:1111:2222:33333::4444:5555]" => "VOID_IPv6_ADDRESS",
            "first.last\@[IPv6:::12.34.56.78]" => "",
            "first.last\@[IPv6:1111:2222:3333:4444:5555:6666:12.34.56.78]" => "VOID_IPv6_ADDRESS",
            "first.last\@[IPv6:1111:2222:3333:4444:5555:6666:7777:8888:9999]" => "VOID_IPv6_ADDRESS",
            "first.last\@[IPv6:1111:2222:3333:4444:5555:6666:7777:8888]" => "",
            "first.last\@[IPv6:1111:2222:3333:4444:5555:6666:7777]" => "VOID_IPv6_ADDRESS",
            "first.last\@[IPv6:1111:2222:3333:4444:5555:6666::]" => "",
            "first.last\@[IPv6:1111:2222:3333::4444:5555:6666:7777]" => "",
            "first.last\@[IPv6:1111:2222:3333::4444:5555:6666]" => "",
            "first.last\@[IPv6:1111:2222:333x::4444:5555]" => "VOID_IP_ADDRESS",
            "first.last\@[IPv6:1111:2222::3333::4444:5555:6666]" => "VOID_IPv6_ADDRESS",
            "first.last\@[IPv6:::1111:2222:3333:4444:5555:6666]" => "",
            "foobar\@192.168.0.1" => "",
            "foo\@[\\1.2.3.4]" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "test\@123.123.123.123" => "",
            "test\@123.123.123.123]" => "DOMAIN_CHARS_VOID",
            "test\@[123.123.123.123" => "DOMAIN_TLD_ALL_NUMBERS",
            "test\@[123.123.123.123]" => "",

            # quotes
            "\"a,b\"\@epages.de" => "",
            "\"a\\,b\"\@epages.de" => "",
            "\"Abc\@def\"\@example.com" => "",
            "\"Abc\\\@def\"\@example.com" => "",
            "\"Doug \"Ace\" L.\"\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "\"Doug \\\"Ace\\\" L.\"\@example.com" => "",
            "\"Fred Bloggs\"\@example.com" => "",
            "\"Fred\\ Bloggs\"\@example.com" => "",
            "\"Ima.Fool\"\@example.com" => "",
            "\"Joe.\\\\Blow\"\@example.com" => "",
            "\"Joe\\\\Blow\"\@example.com" => "",
            "\"Test \\\"Fail\\\" Ing\"\@example.com" => "",
            "\"[[ test ]]\"\@example.com" => "",
            "\"\"\"\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "\"\"\@example.com" => "ADDR_LOCAL_ATOM_EMPTY",
            "\"\\\"\@example.com" => "ADDR_UNMATCHED_QUOTE",
            "\"first(last)\"\@example.com" => "",
            "\"first..last\"\@example.com" => "",
            "\"first.middle.last\"\@example.com" => "",
            "\"first.middle\".\"last\"\@example.com" => "",
            "\"first\".\"last\"\@example.com" => "",
            "\"first\".\"middle\".\"last\"\@example.com" => "",
            "\"first\".last\@example.com" => "",
            "\"first\".middle.\"last\"\@example.com" => "",
            "\"first\"last\"\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "\"first\@last\"\@example.com" => "",
            "\"first\\\"last\"\@example.com" => "",
            "\"first\\\\\"last\"\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "\"first\\\\\\\"last\"\@example.com" => "",
            "\"first\\\\last\"\@example.com" => "",
            "\"first\\last\"\@example.com" => "",
            "\"hello my name is\"\@stutter.com" => "",
            "\"qu\@example.com" => "ADDR_UNMATCHED_QUOTE",
            "\"test.test\"\@example.com" => "",
            "\"test\"test\"\@example.com" => "ADDR_LOCAL_VOID_CHARS",
            "\"test\@test\"\@example.com" => "",
            "\"test\\\"blah\"\@example.com" => "",
            "\"test\\\\blah\"\@example.com" => "",
            "\"test\\test\"\@example.com" => "",
            "first.\"mid\\dle\".\"last\"\@example.com" => "",
            "first\\\@last\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "first\\\\\@last\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "first\\last\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "ote\"\@example.com" => "ADDR_UNQUOTED_SPECIAL_CHAR",
            "test.\"test\"\@example.com" => "",
        },

    );

    _test_smtp_address($SmtpTests{common}, $SmtpTests{$_}, $_) foreach qw(2821 5321);

    #--------------------------------------------------------------------
    # TestMultipleEMailAddresses (RFC2822)
    #--------------------------------------------------------------------

    my %MultiTests = (
        '"Manuel Blechschmidt" <mblechschmidt@epages.de>;"Thomas; Mller" <tmueller@epages.de>' => '',
        'max.mustermann@epages.de; maria.musterfrau@epages.de' => '',
        'max.mustermann@epages.de' => '',
        'max.mustermann@epages.de; max' => 'DOMAIN_NAME_EMPTY',
        'max.mustermann@epages.de, test@test.de' => '',
        );

    _test_multiple_email_addresses(\%MultiTests);
}

sub _test_multiple_email_addresses {
    my ($TestHash, $Rfc, $Debug) = @_;
    foreach my $Key (sort keys %$TestHash) {
        my $ReturnString = $TestHash->{$Key};
        $ReturnString = 'VALID' unless $ReturnString;
        eval { TestMultipleEMailAddresses($ReturnString, $Key, $Rfc, $Debug); };
        my $Msg = ExistsError() ? GetError->message : 'VALID';
        $Msg =~ s/.*\((.*?)[:\)].*/$1/s;
        my $OK = $TestHash->{$Key} ? $Msg eq $TestHash->{$Key} : !ExistsError();
        ok($OK, "TestMultipleEMailAddresses('$ReturnString', '$Key', ".($Rfc // 'undef').") -> $Msg" );
    }
}

sub _test_smtp_address {
    my ($CommonHash, $RfcHash, $Rfc) = @_;
    my %TestHash = %$CommonHash;
    $TestHash{$_} = $RfcHash->{$_} foreach keys %$RfcHash;
    _test_email_address(\%TestHash, $Rfc);
}

sub _test_email_address {
    my ($TestHash, $Rfc, $Debug) = @_;
    foreach my $Key (sort keys %$TestHash) {
        my $ReturnString = $TestHash->{$Key};
        $ReturnString = 'VALID' unless $ReturnString;
        eval { TestEMailAddress($ReturnString, $Key, $Rfc, $Debug); };
        my $Msg = ExistsError() ? GetError->message : 'VALID';
        $Msg =~ s/.*\((.*?)[:\)].*/$1/s;
        my $OK = $TestHash->{$Key} ? $Msg eq $TestHash->{$Key} : !ExistsError();
        ok($OK, "TestEMailAddress('$ReturnString', '$Key', ".($Rfc // 'undef').") -> $Msg" );

=for fundamental_research
        use Net::SMTP;
        print "\n<Net::SMTP>$Key</>\n";
        my $smtp = Net::SMTP->new('ep-exchange');
        $smtp->mail($Key) or print ".....ERROR.....\n";
        $smtp->reset();

        use Mail::Address;
        print "\n<Mail::Address>$Key</>\n";
        my @oEMailAddress = Mail::Address->parse($Key);

        for (0 .. scalar(@oEMailAddress)-1) {
            print "address: ", $oEMailAddress[$_]->address, "\n";
            print "phrase: ",  $oEMailAddress[$_]->phrase, "\n";
            print "comment: ", $oEMailAddress[$_]->comment, "\n";
        }
=cut
    }
}

sub _extract_addr_spec {
    my ($TestHash, $Rfc) = @_;
    foreach my $Addr (sort keys %$TestHash) {
        my $Key = $Addr;
        my ($Error) = ExtractAddrSpec(\$Addr, $Rfc);
        my $Value = $Error ? $Error : $Addr;
        my $OK = $Value eq $TestHash->{$Key};
        ok($OK, "ExtractAddrSpec('$Key', $Rfc) -> $Value" );
    }
}

RunScript(Sub=>\&Main);
