#!perl -w
use Test::More;
use strict;
use DE_EPAGES::Core::API::PerlTools qw(
    AddDuration
    CheckIpFormat
    Compare
    CopyStructure
    CreateNewPassWD
    CreateSecureCode
    CryptPassword
    ElementOf
    GenerateGUID
    GenerateShortGUID
    GetCallerPackage
    GetIPAddress
    GetMD5Digest
    IndexOf
    Ip2Hex
    VerifySecureCode
    fcmp
    trim
    unique
);
use Data::Dumper qw ( Dumper );
use DateTime;

plan tests => 49;

# GetCallerPackage
ok( Test() eq 'main', 'GetCallerPackage main');
ok( MyPackage::Test()  eq 'MyPackage',  'GetCallerPackage MyPackage');
ok( MyPackage::Test2() eq 'MyPackage2',  'GetCallerPackage MyPackage2' );

# trim, unique
ok( ('AAC'          eq join '', trim( "  A", " A", "\t  C\n" )), "trim" );
ok( ("  A A\t  C\n" eq join '', unique( "  A", " A", "\t  C\n" )), "unique" );
ok( ('AAC'          eq join '', trim unique( "  A", " A", "\t  C\n" )), "trim unique" );
ok( ('AC'           eq join '', unique trim( "  A", " A", "\t  C\n" )), "unique trim" );

# test sha and des password encryption
my $Password = "hallo";
ok( length(CryptPassword($Password)) == 64, "SHA Passwords if no salt is defined");
ok( length(CryptPassword($Password, "1234567890123")) == 13, "DES passwords if salt is defined as old DES code");
ok( length(CryptPassword($Password, "1234")) == 64, "SHA passwords if salt no old DES code");


# CopyStructure
my $Structure1 = GetStructure();
my $Structure2 = GetStructure();
local $Data::Dumper::Sortkeys = 1;
ok (Dumper($Structure1) eq Dumper(CopyStructure($Structure2)), 'structure equal');

# CheckIpFormat
ok( !defined CheckIpFormat('1.0.0.127'), "CheckIpFormat IP4");
ok( !defined CheckIpFormat('2001:4bd8:2:2:250:56ff:fe05:299'), "CheckIpFormat IPv6");
ok( !defined CheckIpFormat('::1'), "CheckIpFormat localhost IPv6");
cmp_ok( CheckIpFormat('100127'), 'eq', 'VOID_IP_ADDRESS', "CheckIpFormat - no dots");
cmp_ok( CheckIpFormat('x.0.0.127'), 'eq', 'VOID_IP_ADDRESS', "CheckIpFormat - invalid characters");
cmp_ok( CheckIpFormat('1.0..0.127'), 'eq', 'VOID_IPv4_ADDRESS', "CheckIpFormat - invalid IPv4");
cmp_ok( CheckIpFormat('1:1:2'), 'eq', 'VOID_IPv6_ADDRESS', "CheckIpFormat - invalid IPv6");

# GetIPAddress
my $IP = GetIPAddress();
ok( !defined CheckIpFormat('::1'), "GetIPAddress: $IP");

# IP2Hex
cmp_ok( Ip2Hex('1.0.0.127'), 'eq', '0100007F', "Ip2Hex localhost IP4");
cmp_ok( Ip2Hex('::1'), 'eq', '00000000000000000000000000000001',  "Ip2Hex localhost IPv6");
cmp_ok( Ip2Hex('2001:4bd8:2:2:250:56ff:fe05:299'), 'eq', '20014BD800020002025056FFFE050299', "Ip2Hex IPv6");
cmp_ok( Ip2Hex('2001:4bd8:2:2:250:56ff:fe05:299', 8), 'eq', 'FE050299', "short Ip2Hex IPv6");

# GenerateGUID
my $GUID1 = GenerateGUID();
ok ($GUID1 =~ /^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12}$/, 'GUID format');

my $findDoubleGUID = 1;
my %GUIDs;
for(my $i=0;$i<100000;$i++) {
    my $GUID = GenerateGUID();
    $findDoubleGUID = 0 if exists $GUIDs{$GUID};
    $GUIDs{$GUID} = undef;
    use DE_EPAGES::Core::API::PerlTools;
}
ok ($findDoubleGUID , '100000 created GUIDs not equal');

# GenerateShortGUID
my $ShortGUID = GenerateShortGUID();
ok ($ShortGUID =~ /^[0-9A-F]{20}$/, 'GenerateShortGUID format');

# fcmp
ok( ( ElementOf('ten', ('twelve', 'eleven', 'ten') )), "ElementOf");
ok( ( 2 == IndexOf('ten', ('twelve', 'eleven', 'ten') )), "IndexOf");
ok( 0  == fcmp(0.00011,1.10 - 1.09989), 'fcmp real');
ok( -1 == fcmp(0,1), 'fcmp(0,1)');
ok( 0   == fcmp(0.00011,0.00012, 4), 'fcmp equal with 4 decimal digits');
ok( -1  == fcmp(0.00011,0.00012, 5), 'fcmp less with 5 decimal digits');

# CreateNewPassWD
ok(length(CreateNewPassWD) == 8, 'Password with right length');
ok(length(CreateNewPassWD('', 7)) == 7, 'Password with right length');
ok(CreateNewPassWD =~ m/^[A-Za-z0-9]/, 'correct characters');

# Compare
ok( 0 == Compare( 'String', undef, undef ), "Compare undef to empty string" );
ok( -1 == Compare( 'String', undef, '' ), "Compare undef to empty string" );
ok( 1 == Compare( 'String', '', undef ), "Compare undef to empty string" );
ok( 1 == Compare( 'String', 'a', 'B' ), "Compare case sensitive" );

is( GetMD5Digest(\"test"), "098f6bcd4621d373cade4e832627b4f6", "GetMD5Digest" );    #"

# CreateSecureCode, VerifySecureCode
is( CreateSecureCode("m\x{F6}llerfranzmoellder\@epages.de"), '218aec30aca416ddc49e04b4f903227c', 'CreateSecureCode' );
is( VerifySecureCode("m\x{F6}llerfranzmoellder\@epages.de", '218aec30aca416ddc49e04b4f903227c'), 1, 'VerifySecureCode - true' );
is( VerifySecureCode("m\x{F6}llerfranzmoellder\@epages.de", 'cc9e0e7699711ccb4d9e077690076a52'), 0, 'VerifySecureCode - false' );

# AddDuration
my $dt = DateTime->new( year => 2010, month => 3, day => 28, hour => 0, minute => 30, time_zone => 'Europe/Berlin' );
my $dt2 = AddDuration( $dt, 'hours', -2 );
cmp_ok( "$dt",  'eq', '2010-03-28T00:30:00', "original value has not changed" );
cmp_ok( "$dt2", 'eq', '2010-03-27T22:30:00', "substract 2 hours" );
cmp_ok( $dt2->time_zone->name, 'eq', 'Europe/Berlin', "time zone does not change" );

my $dt3 = AddDuration( $dt, 'hours', 2 );
cmp_ok( "$dt3", 'eq', '2010-03-28T03:30:00', "skip invalid hour during daylight saving time switch" );

$dt = DateTime->new( year => 2010, month => 3, day => 28, hour => 0, minute => 30 );
my $dt4 = AddDuration( $dt, 'hours', 2 );
cmp_ok( "$dt4", 'eq', '2010-03-28T02:30:00', "add 2 hours to floating time" );
cmp_ok( $dt4->time_zone->name, 'eq', 'floating', "floating time zone does not change" );


# -----------------------------------------------------------------------------
# helper functions

sub Test {
    my $MyPackage = GetCallerPackage();
    return $MyPackage;
}
sub GetStructure {
    return {
        'Test1' => [1,3,4,5,6],
        'Test2' => {
            'AA' => 2,
            'BB' => 5,
        },
    };
}

##no critic (ProhibitMultiplePackages)
package MyPackage;
use DE_EPAGES::Core::API::PerlTools qw( GetCallerPackage );
sub Test {
    my $MyPackage = GetCallerPackage();
    return $MyPackage;
}
sub Test2 { return MyPackage2::Test(); }

package MyPackage2;
use DE_EPAGES::Core::API::PerlTools qw( GetCallerPackage );
sub Test {
    my $MyPackage = GetCallerPackage();
    return $MyPackage;
}
##use critic

