use strict;
use Test::More tests => 42;
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite GetCurrentDBHandle );

sub Main {
    ConnectTestSite();

    my $dbi = GetCurrentDBHandle();
    my $driverName = $dbi->driverName;
    ok( $dbi, "GetCurrentDBHandle driverName=".$driverName );

    ok( Init( $dbi ), "Init" );

    ok( $dbi->existsTable( 'dbitest', 'name' ), "existsTable: dbitest exists" );
    ok( !$dbi->existsTable( 'notable', 'wrong' ), "existsTable: notable does not exist" );
    ok( $dbi->existsTableColumn( 'dbitest', 'name' ), "existsTableColumn: dbitest.name exists" );
    ok( !$dbi->existsTableColumn( 'dbitest', 'nocolumn' ), "existsTableColumn: dbitest.nocolumn does not exist" );

    $dbi->do( 'INSERT INTO dbitest (id, name) VALUES (?, ?)', [10, 'Name1'] );
    my $Name = $dbi->execute( 'SELECT name FROM dbitest WHERE id=?', [10] )->[0]->[0];
    ok( $Name eq 'Name1', "basic INSERT and SELECT" );

    $dbi->do( 'UPDATE dbitest SET name=? WHERE id=?', ['Name2', 10] );
    $Name = $dbi->execute( 'SELECT name FROM dbitest WHERE id=?', [10] )->[0]->[0];
    ok( $Name eq 'Name2', "simple update" );

    my $Now = $dbi->currentDateTime;
    ok( $Now->year > 2003, "currentDateTime" );

    my $dt = $dbi->execute( 'SELECT dt FROM dbitest WHERE id=?', [10], ['DateTime'] )->[0]->[0];
    ok( !defined $dt, "select NULL value" );

    $dbi->do( 'UPDATE dbitest SET dt=? WHERE id=?', [$Now, 10] );
    $dt = $dbi->execute( 'SELECT dt FROM dbitest WHERE id=?', [10], ['DateTime'] )->[0]->[0];
    ok( $dt->hms eq $Now->hms, "equal time" );
    ok( $dt->ymd eq $Now->ymd, "equal date" );

# does not work for Sybase, because nanoseconds may be different
#    ok( $dt eq $Now, "update DATETIME value" );

    $dbi->do( 'DELETE FROM dbitest WHERE id=?', [10] );
    my $aRows = $dbi->execute( 'SELECT name FROM dbitest WHERE id=?', [10] );
    ok( 0 == scalar @$aRows, "deleted row" );
    $dbi->addTableColumn('dbitest', 'testcolumn varchar(30) NULL');
    ok( $dbi->existsTableColumn('dbitest', 'testcolumn'), "added column testcolumn" );

    SKIP: {
        skip( "modifyTableColumn is not supported by SQLite", 2 ) if $driverName eq 'SQLite';

        $dbi->modifyTableColumn('dbitest', 'testcolumn varchar(30) NOT NULL');
        ok( 1, 'testcolumn modified' );
        $dbi->modifyTableColumn('dbitest', 'testcolumn nvarchar(30) NOT NULL');
        ok( 1, 'testcolumn modified' );
    }

    my $BoolType = $driverName eq 'Sybase' ? 'BIT'
                 : $driverName eq 'mysql' ? 'BOOL'
                 : 'TINYINT';
    $dbi->addTableColumn('dbitest', "isactive $BoolType default 0 not null");
    ok( 1, 'isactive bit default 0 not null added' );

    SKIP: {
        skip( "modifyTableColumnDropDefault is not supported by SQLite", 1 ) if $driverName eq 'SQLite';

        $dbi->modifyTableColumnDropDefault('dbitest', 'isactive' );
        ok( 1, 'drop default isactive = 0' );
    }

    SKIP: {
        skip( "dropTableColumn is not supported by SQLite", 1 ) if $driverName eq 'SQLite';

        $dbi->dropTableColumn('dbitest', 'testcolumn');
        ok( !$dbi->existsTableColumn('dbitest', 'testcolumn'), 'testcolumn dropped' );
    }

    # create and drop indexes
    ok( !$dbi->existsIndex('dbitest', 'i_dbitest_name_dt'), "dbitest.i_dbitest_name_dt does not exist" );
    $dbi->createIndex('dbitest', 'i_dbitest_name_dt', undef,
           ['name', 'dt'], {'name' => 'DESC', 'dt' => 'ASC'});
    ok( $dbi->existsIndex('dbitest', 'i_dbitest_name_dt'), "dbitest.i_dbitest_name_dt created" );

    ok( !$dbi->existsIndex('dbitest', 'u_dbitest_name'), "dbitest.i_dbitest_name_dt does not exist" );
    $dbi->createIndex('dbitest', 'u_dbitest_name', 'UNIQUE', ['name']);
    ok( $dbi->existsIndex('dbitest', 'u_dbitest_name'), "dbitest.u_dbitest_name created" );

    $dbi->dropIndex('dbitest', 'i_dbitest_name_dt');
    ok( !$dbi->existsIndex('dbitest', 'i_dbitest_name_dt'), "dbitest.i_dbitest_name_dt deleted" );

    $dbi->dropIndex('dbitest', 'u_dbitest_name');
    ok( !$dbi->existsIndex('dbitest', 'u_dbitest_name'), "dbitest.u_dbitest_name deleted" );

    my $ProcParameter = $dbi->getParameterString({'Name' => 'String;NotNull',
                            'Descr' => 'String',
                            'AnyNumber' => 'Integer'
                        }, {'Name'=>'Test\'Name','AnyNumber'=> 1});
    ok ($ProcParameter eq "\@AnyNumber=1, \@Name='Test\'\'Name'", 'getParameterString');

    my $TestString = "select \0 from dbitest";
    ok( $TestString =~ /\0/, "teststring 0" );

    eval { $dbi->getParameterString({'SearchName'=>'String;NotNull'}, { 'SearchName' => $TestString }); };
    ok(ExistsError() && (GetError()->code eq 'STRING_CONTAINS_0x0'), 'raise error if null terminated string');

    # temporary table
    my $TempTableName = $dbi->tempTableName('temptable');
    $dbi->createTempTable('temptable (id INTEGER NOT NULL, value VARCHAR(255))');
    # ok( $dbi->existsTable($TempTableName), 'temporary table created' );

    # test that the temporary table actually works
    $dbi->do("INSERT INTO $TempTableName (id, value) VALUES (25, 'testvalue')");
    my $TempValue = $dbi->execute("SELECT value FROM $TempTableName WHERE id = 25")->[0]->[0];
    ok( $TempValue eq 'testvalue', "temporary table works" );

    $dbi->dropTable($TempTableName);
    #ok( !$dbi->existsTable($TempTableName), 'temporary table deleted' );

    my $DriverName = $dbi->driverName;
    print "# Driver: $DriverName\n";
    my $MoneyType = $DriverName eq 'Sybase' ? 'MONEY' : 'DECIMAL(10,4)';
    my $FloatType = $DriverName eq 'Sybase' ? 'FLOAT' : 'DOUBLE';

    $TempTableName = $dbi->tempTableName('temptable');
    $dbi->createTempTable(<<END_SQL);
temptable (
    id INT NOT NULL,
    floatvalue $FloatType NULL,
    moneyvalue $MoneyType NULL,
    intvalue INT NULL
)
END_SQL

    $dbi->do("INSERT INTO $TempTableName (id, floatvalue, moneyvalue, intvalue) VALUES (1, 1234.12345, 1234.1234, 1234)");
    $dbi->do("INSERT INTO $TempTableName (id, floatvalue, moneyvalue, intvalue) VALUES (2, 1234.1, 1234.1, 1234)");
    $dbi->do("INSERT INTO $TempTableName (id, floatvalue, moneyvalue, intvalue) VALUES (3, 1234, 1234, 1234)");
    $dbi->do("INSERT INTO $TempTableName (id, floatvalue, moneyvalue, intvalue) VALUES (4, NULL, NULL, NULL)");
    my $aaRows = $dbi->execute("SELECT id, floatvalue, moneyvalue, intvalue FROM $TempTableName ORDER BY id",
        [], [qw(Integer Float Float Integer)], [qw(ID FloatValue MoneyValue IntValue)]);
    cmp_ok( $aaRows->[0]->{'FloatValue'}, 'eq', '1234.12345', 'Float 1234.12345' );
    cmp_ok( $aaRows->[0]->{'IntValue'}, 'eq', '1234', 'Integer 1234' );
    cmp_ok( $aaRows->[0]->{'MoneyValue'}, 'eq', '1234.1234', 'Money 1234.1234' );
    cmp_ok( $aaRows->[1]->{'FloatValue'}, 'eq', '1234.1', 'Float 1234.1' );
    cmp_ok( $aaRows->[1]->{'MoneyValue'}, 'eq', '1234.1', 'Money 1234.1' );
    cmp_ok( $aaRows->[2]->{'FloatValue'}, 'eq', '1234', 'Float 1234' );
    cmp_ok( $aaRows->[2]->{'MoneyValue'}, 'eq', '1234', 'Money 1234' );

    $dbi->dropTable($TempTableName);

    my ($Database, $User, $Password) = $dbi->createUserAndStoreDBName("StoreES2");
    is( $Database, 'storees2db', "createUserAndStoreDBName database" );
    is( $User, 'storees2', "createUserAndStoreDBName user" );
    is( $Password, 'epages', "createUserAndStoreDBName (long) password" );

    ($Database, $User) = $dbi->createUserAndStoreDBName("Storexxxxxxxxxxxxxxxxxxx24");
    is( $Database, 'storexxxxxxxxxxxxxxxxxxx24db', "createUserAndStoreDBName (long) database" );
    if( $DriverName eq 'mysql' ) {
        is( $User, 'storexxxxxxxxx', "createUserAndStoreDBName (long) user" );
    }
    else {
        is( $User, 'storexxxxxxxxxxxxxxxxxxx24', "createUserAndStoreDBName (long) user" );
    }

    ok( Cleanup( $dbi ), "Cleanup" );

    DisconnectSite();
}

sub Init {
    my ( $dbi ) = @_;
    Cleanup( $dbi );

    $dbi->do( "CREATE TABLE dbitest( id INTEGER, name VARCHAR(50) NULL, dt DATETIME NULL )" );

    return $dbi->existsTableColumn( 'dbitest', 'name' );
}

sub Cleanup {
    my ( $dbi ) = @_;

    $dbi->do( "DROP TABLE dbitest" ) if $dbi->existsTable( 'dbitest' );
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
