use Test::More;
use strict;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::PerlTools qw ( ElementOf GenerateGUID );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite  );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByPath LoadRootObject LoadObjectByPath );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Product::API::Shop qw ( GetProduct );
use DE_EPAGES::FactFinder::API::FactFinder qw ( TriggerExtraSchedule InitShop DeleteShop GetAllChannels ExistsChannel ExistsUser GetAllUsers GetVisibleLanguages GetRecommendation );
use DE_EPAGES::FactFinder::API::FactFinderExport qw ( ExportAll ExportUpdateTrigger );
use DE_EPAGES::FactFinder::API::Table::FactFinderShopMap qw ( ExistsFactFinderShopMap );

my $ROOTPATH = '/Test';

sub testFactFinder {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            plan skip_all => "factfinder disabled." unless (LoadRootObject()->get("FactFinder_Enabled"));
            ok(&Init,'STARTUP');
            my $Shop      = LoadObjectByPath( $ROOTPATH . '/TestShopFactFinder' );
            plan skip_all => 'shop init failed.' unless (defined $Shop);

            my $InitSuccess = &TestInitFactFinder($Shop);

            eval {
                SKIP: {
                    skip('fact-finder init failed.', 3) unless ($InitSuccess);
                    # test regular export
                    &TestExportProducts($Shop,0);
                    # test export trigger
                    &TestExportProducts($Shop,1);
                    # test recommendation engine
                    &TestRecommendation($Shop);
                }
            };
            if (ExistsError()) {
                my $Error = GetError();
                LogError($Error);
                fail('ERROR: '.$Error->shortMessage);
            }

            &TestDeleteFactFinder($Shop) if ($InitSuccess);
            ok(&Cleanup, 'CLEANUP');

            done_testing();
        });
}

sub TestExportProducts {
    my ($Shop, $UpdateTrigger) = @_;
    my $Shops = [$Shop];
    my $NoOutput = 0;
    my $NoUpload = 0;
    my $hExport;

    if ($UpdateTrigger) {
        TriggerExtraSchedule($Shop);
        $hExport = ExportUpdateTrigger($Shops, $NoUpload, $NoOutput);
    } else {
        $hExport = ExportAll($Shops, $NoUpload, $NoOutput);
    }

    ok(defined $hExport &&
        -e ($hExport->{'Files'}->{$Shop->alias})
        , "EXPORT: product export file generated".($UpdateTrigger?'(triggered)':''));
    fail("EXPORT: Upload failed: ".($UpdateTrigger?'(triggered) ':'')
        . ($hExport->{'Errors'}->{$Shop->alias})) if (defined $hExport->{'Errors'}->{$Shop->alias});
    my $ChannelDE = $Shop->get('FactFinder_Channel',1);
    ok(defined($ChannelDE)
        && defined($ChannelDE->get('LastExport'))
        , "EXPORT: last export date is set. ".($UpdateTrigger?'(triggered)':''));

}

sub TestRecommendation {
    my ($Shop) = @_;
    my $Channel = $Shop->get('FactFinder_Channel',1);
    unless (defined $Channel) {
        fail('RECOMMENDATION: channel de not existing.');
        return;
    }
    my $ChannelDE = $Channel->alias;

    my $Remote = DE_EPAGES::FactFinder::API::Object::RemoteFactFinder->factory($Shop);
    $Remote->_isLogCommunication(1);

    # fill recommendation engine.
    for (my $i=1; $i <= 5 ; ++$i) {
        my $GUID = GenerateGUID;
        foreach my $Event (qw( cart checkout )) {

            $Remote->logInformation($ChannelDE,{
                'channel' => $ChannelDE,
                'event'   => $Event,
                'sid'     => $GUID,
                'id'      => 'lt_0401107001',
                'count'   => 2,
                'price'   => 31.01
            });

            $Remote->logInformation($ChannelDE,{
                'channel' => $ChannelDE,
                'event'   => $Event,
                'sid'     => $GUID,
                'id'      => 'cg_0101104470',
                'count'   => $i,
                'price'   => 23.01
            });

            $Remote->logInformation($ChannelDE,{
                'channel' => $ChannelDE,
                'event'   => $Event,
                'sid'     => $GUID,
                'id'      => 'wb_1003111010',
                'count'   => $i+2,
                'price'   => 13.01
            });
        }
    }

    pass('RECOMMENDATION: passed logInformation');
    $Remote->buildMatrix($ChannelDE);
    pass('RECOMMENDATION: passed BuildMatrix');
    my $Product = GetProduct($Shop, 'cg_0101104470');
    my $aRecommendedProducts = GetRecommendation ($Product, $Channel, $Remote);
    cmp_ok(scalar @$aRecommendedProducts, '>', 0 , 'RECOMMENDATION: recommendation engine working.');
}

sub TestInitFactFinder {
    my $Shop = shift;
    my @keys = qw( FactFinder_IsInitialized FactFinder_Account FactFinder_PasswordMD5 FactFinder_BaseURL );
    eval {

        # test import.
        ok(!ExistsFactFinderShopMap($Shop->id),'INIT: shopmap entry empty.');
        InitShop($Shop,0);
        ok(ExistsFactFinderShopMap($Shop->id),'INIT: shopmap entry exists.');

        # test shop attributes
        my $hInfo = $Shop->get(\@keys);
        ok (defined ($hInfo->{$_}), 'INIT: shop attribute '.$_) foreach (@keys);

        # test user:
        my @RemoteUsers = map { $_->{name} } @{ GetAllUsers($Shop)};
        ok(ElementOf($hInfo->{FactFinder_Account},@RemoteUsers), "OK: remote user=".$hInfo->{FactFinder_Account});
        ok(ExistsUser($Shop),"OK: remote user via ExistsUser.");

        # test channels
        my $Folder = $Shop->folder('FactFinderChannels');
        ok(defined $Folder, "INIT: folder");
        my $Channels = $Folder->children;
        my $VisibleLanguages = scalar @{GetVisibleLanguages($Shop)};
        ok(scalar(@$Channels) == $VisibleLanguages, "INIT: channels=".$VisibleLanguages);

        # test de channel
        my $Channel  = $Shop->get('FactFinder_Channel',1);
        ok(defined $Channel, 'INIT: channel_de exists.');

        # test remote channels.
        my $RemoteChannels = GetAllChannels($Shop);

        foreach my $LocalChannel (@$Channels){
            ok(ElementOf($LocalChannel->alias,@$RemoteChannels), "OK: remote channel=". $LocalChannel->alias);
            ok(ExistsChannel($Shop, $LocalChannel), "OK: remote channel via ExistsChannel");
        }
    };

    if (ExistsError()) {
        my $Error = GetError();
        LogError($Error);
        fail('INIT: shop attribute '.$_) foreach(@keys);
        fail("INIT: folder");
        fail("INIT: channels");
        return 0;
    }
    return 1;
}

sub TestDeleteFactFinder {
    my $Shop = shift;
    my @keys = qw( FactFinder_IsInitialized FactFinder_Account FactFinder_PasswordMD5  );

    eval {
        DeleteShop($Shop);
        ok(!ExistsFactFinderShopMap($Shop->id),'DEL: shopmap entry empty again.');
        my $hInfo = $Shop->get(\@keys);
        ok (! $hInfo->{$_}, 'DEL: no more shop attribute '.$_) foreach (@keys);
        my $Folder = $Shop->folder('FactFinderChannels');
        my $Channels = $Folder->children;
        ok(scalar (@$Channels) == 0, "DEL: no more channels");
    };
    if (ExistsError()) {
        fail('DEL: no more shop attribute');
        fail('DEL: no more channels');
        my $Error = GetError();
        LogError($Error);
    }
}

sub Main {
    &testFactFinder;
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript( Sub => \&Main );

1;
