use strict;
use Test::More tests => 39;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);

sub TestForum {
    my ($Shop) = @_;

    my $Forum = $Shop->get('Categories')->insertChild('Forum', {
        Alias => 'Forum',
        IsVisible => 1,
        VisibleInNavigation => 1,
    });

    my $PostClass = $Forum->getPostClassName;

    my $Thread = $Forum->insertChild( $PostClass, {
        Subject           => 'Subject',
        Message           => 'Message',
    });

    # test defaults
    is( $Thread->get('Forum')->id, $Forum->id, "Thread.Forum default" );
    is( $Thread->get('Thread')->id, $Thread->id, "Thread.Thread default" );

    is( $Thread->get('RemoteIP'), undef, "Thread.RemoteIP default" );
    is( $Thread->get('IsVisible'), 0, "Thread.IsVisible default" );
    is( $Thread->get('IsClosed'), 0, "Thread.IsClosed default" );
    is( $Thread->get('Spam'), 0, "Thread.Spam default" );
    is( $Thread->get('User'), undef, "Thread.User default" );

    # test attributes
    is( $Thread->get('Subject'), 'Subject', "Thread.Subject" );
    is( $Thread->get('Message'), 'Message', "Thread.Message" );

    $Thread->set({ 'IsVisible' => 1 });
    is( $Thread->get('IsVisible'), 1, "Thread.IsVisible updated" );

    my $Reply = $Thread->insertChild( $PostClass, {
        Subject   => 'Re: ' . $Thread->get('Subject'),
        Message   => 'Reply Message',
        IsVisible => 1,
        RemoteIP => '2001:fecd:ba23:cd1f:dcb1:1010:9234:4088',
    });

    my $Reply2= $Thread->insertChild( $PostClass, {
        Subject   => 'Re: ' . $Thread->get('Subject'),
        Message   => 'Reply Message',
        IsVisible => 0,
        RemoteIP => '2001:fecd:ba23:cd1f:dcb1:1010:111.111.111.111',
    });

    # test attributes
    is( $Reply->get('Subject'), 'Re: Subject', "Reply.Subject" );
    is( $Reply->get('Message'), 'Reply Message', "Reply.Message" );
    is( $Reply->get('Forum')->id, $Forum->id, "Reply.Forum default" );
    is( $Reply->get('Thread')->id, $Thread->id, "Reply.Thread default" );
    is( $Reply->get('RemoteIP'), '2001:fecd:ba23:cd1f:dcb1:1010:9234:4088', "Thread.RemoteIP IPv6" );
    is( $Reply2->get('RemoteIP'), '2001:fecd:ba23:cd1f:dcb1:1010:111.111.111.111', "Thread.RemoteIP IPv6 mixed mode" );

    is( scalar @{ $Forum->get('Posts') }, 3, "number of Forum.Posts" );
    is( scalar @{ $Forum->get('VisiblePosts') }, 2, "number of visible Forum.Posts" );
    is( scalar @{ $Forum->get('Threads') }, 1, "number of Forum.Threads" );
    is( scalar @{ $Forum->get('VisibleThreads') }, 1, "number of visible Forum.VisibleThreads" );
    is( scalar @{ $Forum->get('Users') }, 0, "number of Forum.Users" );
}

sub TestGuestBook {
    my ($Shop) = @_;

    my $GuestBook = $Shop->get('Categories')->insertChild('GuestBook', {
        Alias => 'GuestBook',
        IsVisible => 1,
        VisibleInNavigation => 1,
    });

    my $PostClass = $GuestBook->getPostClassName;

    my $Entry_1 = $GuestBook->insertChild( $PostClass, {
        Subject => 'Subject 1',
        Message => 'Message 1',
    });

    # test defaults
    is( $Entry_1->get('Forum')->id, $GuestBook->id, "GuestBookEntry.Forum default" );
    is( $Entry_1->get('Thread')->id, $Entry_1->id, "GuestBookEntry.Thread default" );
    is( $Entry_1->get('IsVisible'), 0, "GuestBookEntry.IsVisible default" );
    is( $Entry_1->get('IsClosed'), 0, "GuestBookEntry.IsClosed default" );
    is( $Entry_1->get('Spam'), 0, "GuestBookEntry.Spam default" );
    is( $Entry_1->get('User'), undef, "GuestBookEntry.User default" );

    # test attributes
    is( $Entry_1->get('Subject'), 'Subject 1', "GuestBookEntry.Subject" );
    is( $Entry_1->get('Message'), 'Message 1', "GuestBookEntry.Message" );

    my $Entry_2 = $GuestBook->insertChild( $PostClass, {
        Subject => 'Subject 2',
        Message => 'Message 2',
        IsVisible => 1,
    });
    is( $Entry_2->get('IsVisible'), 1, "GuestBookEntry.IsVisible" );

    is( scalar @{ $GuestBook->get('Posts') }, 2, "GuestBook.Posts" );
    is( scalar @{ $GuestBook->get('VisiblePosts') }, 1, "GuestBook.Posts" );
    is( scalar @{ $GuestBook->get('Threads') }, 2, "GuestBook.Threads" );
    is( scalar @{ $GuestBook->get('VisibleThreads') }, 1, "GuestBook.VisibleThreads" );
    is( scalar @{ $GuestBook->get('Users') }, 0, "GuestBook.Users" );
}

sub Main {
    ok( Init(), 'initialization' );

    my $Test = LoadObjectByPath('Test');
    my $Shop = $Test->insertChild('Shop', {
        Alias => 'TestShop',
        InitialShopWizardStartup => 0
    });

    $Shop->featureMaxValue( LoadObjectByPath('/Features/Forum'), 10 );
    $Shop->featureMaxValue( LoadObjectByPath('/Features/Guestbook'), 10 );

    TestForum( $Shop );
    TestGuestBook( $Shop );

    is( $Shop->featureCurrentValue( LoadObjectByPath('/Features/Forum') ), 1, "Count feature Forum" );
    is( $Shop->featureCurrentValue( LoadObjectByPath('/Features/Guestbook') ), 1, "Count feature GuestBook" );

    ok( Cleanup(), 'cleanup');
}

sub Cleanup {
    LoadObjectByPath('/Test')->delete if ExistsObjectByPath('/Test');
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    return 1;
}

RunScript( Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
