#!perl -w
use strict;
use Test::Simple tests => 44;
use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw (RunOnStore GetTestSite);
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadClassByAlias LoadObjectByPath ExistsObjectByPath );
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode );

my $ROOTPATH = '/Test';

sub Main {
    # create test classes
    ok( Init(), "initialization" );

    # class Locale
    my $Locale_de_DE = LoadClassByAlias('Locale')->loadObject('de_DE');
    ok( $Locale_de_DE->get('Country')->{'Code3'} eq "DEU", "Locale.Country.Code3" );
    ok( $Locale_de_DE->get('Currency')->{'NumCode'} == 978, "Locale.Currency.NumCode" );
    ok( $Locale_de_DE->get('Language')->{'Code3'} eq 'ger', "Locale.Language.Code3" );

    my $Folder = LoadObjectByPath( $ROOTPATH );
    my $Site = $Folder->insertChild('Site', { Alias => 'TestSite' });
    ok( 0 == scalar @{$Site->get('SiteLocales')}, "new site has 0 locales" );

    # attribute Site.Locales
    my $aLocales = $Site->get('Locales');
    ok( scalar @$aLocales >= 2, "some locales exist" );
    ok( scalar @$aLocales == scalar @{LoadRootObject()->get('Locales')}, "Site.Locales==System.Locales" );
    ok( $aLocales->[0]->class->get('Alias') eq 'Locale', "class of Site.Locales[0]" );

    my $IsUsed = 1;
    foreach my $Locale (@$aLocales) {
        $IsUsed = 0 unless $Locale->get('IsUsed');
    }
    ok( $IsUsed, "Site.Locales: all locales are usable" );

    # BackofficeLocales should include all locales set to IsUsed=1
    my %BOLocales = map { $_->get('LocaleID') => 1 } @{$Site->get('BackofficeLocales')};
    ok( exists $BOLocales{'de_AT'}, "de_AT exists in BackofficeLocales" );
    ok( exists $BOLocales{'en_AU'}, "en_AU exists in BackofficeLocales" );

    my $SiteLocale = $Site->addLocale('de_DE');
    ok( $Site->existsLocale('de_DE'), "addLocale de_DE" );
    ok( $SiteLocale->get('LocaleID') eq 'de_DE', "addLocale returns SiteLocale" );

    $Site->addLocale('en_GB');
    ok( $Site->existsLocale('en_GB'), "addLocale en_GB" );

    ok( 2 == scalar @{$Site->get('SiteLocales')}, "site now has 2 locales" );

    $Site->removeLocale('en_GB');
    ok( !$Site->existsLocale('en_GB'), "removeLocale en_GB" );
    ok( 1 == scalar @{$Site->get('SiteLocales')}, "site now has 1 locale left" );

    eval { $Site->addLocale('en_UK'); };
    ok( $@, "non-existing locale" );

    ok( $Site->existsSiteLocale('en_GB'), "existsSiteLocale en_GB" );
    ok( !$Site->existsSiteLocale('en_UK'), "existsSiteLocale en_UK" );

    $SiteLocale = $Site->siteLocale('en_GB');
    ok( $SiteLocale->get('LocaleID') eq 'en_GB', "siteLocale en_GB" );

    my $OrgLocaleID = $Site->get('LocaleID');
    $Site->set({ 'LocaleID' => 'en_AU' });
    ok( $Site->get('SiteLocale')->get('LocaleID') eq 'en_AU', "Site.SiteLocale" );


    my $LocaleClass = LoadClassByAlias('Locale');
    my $Locale_xx_YY = $LocaleClass->insertObject({
        LocaleID => 'xx_YY',
        CountryID => 276,
        CurrencyID => 'EUR',
        LanguageID => GetPKeyLanguageByCode('de'),
    });
    ok( $Locale_xx_YY->get('LocaleID') eq 'xx_YY', "Locale->insertObject" );
    ok( !$Locale_xx_YY->get('IsUsed'), "Locale->get(IsUsed) == 0" );
    ok( !$Site->existsSiteLocale('xx_YY'), "not existsSiteLocale xx_YY, because IsUsed==0" );

    eval { $Site->addLocale('xx_YY'); };
    if( ExistsError() ) {
        my $Error = GetError();
        ok( $Error->code eq 'LocaleDisabled', "add disabled locale" );
    }
    else {
        ok( 0, "addLocale xx_YY should throw error LocaleDisabled" );
    }
    ok( !$Site->existsLocale('xx_YY'), "not existsLocale xx_YY, because locale is disabled" );
    ok( !$Site->existsSiteLocale('xx_YY'), "not existsSiteLocale xx_YY, because locale is disabled" );

    eval { $SiteLocale = $Site->siteLocale('xx_YY'); };
    ok( ExistsError() && GetError->code eq 'LocaleDisabled', "siteLocale xx_YY" );

    $Locale_xx_YY->set({'IsUsed' => 1});
    ok( $Locale_xx_YY->get('IsUsed'), "Locale->get(IsUsed) == 1" );
    ok( $Site->existsSiteLocale('xx_YY'), "existsSiteLocale xx_YY, because IsUsed==1" );
    ok( !$Site->existsLocale('xx_YY'), "not existsLocale xx_YY, because not added" );

    eval { $Site->addLocale('xx_YY'); };
    if( ExistsError() ) {
        ok( 0, "addLocale xx_YY: ". GetError()->message );
    }
    else {
        ok( 1, "addLocale xx_YY" );
    }
    ok( $Site->existsLocale('xx_YY'), "existsLocale xx_YY" );
    ok( $Site->existsSiteLocale('xx_YY'), "existsSiteLocale xx_YY" );

    $Site->set({ 'LocaleID' => 'xx_YY' });
    eval { $Locale_xx_YY->delete; };
    ok( ExistsError(), "can't delete Locale, because it is the default in some site");
    ok( GetError->code eq 'LocaleInUse', "Error->code eq 'LocaleInUse'");

    $Site->set({ 'LocaleID' => $OrgLocaleID });
    $Locale_xx_YY->delete;
    ok( !$Site->existsLocale('xx_YY'), "not existsLocale xx_YY" );
    ok( !$Site->existsSiteLocale('xx_YY'), "not existsSiteLocale xx_YY" );

    # DefaultPageSize
    my $OrgDefaultPageSize = $Site->get('DefaultPageSize');
    ok( 10 == $OrgDefaultPageSize, "default value of DefaultPageSize" );

    $Site->set({ 'DefaultPageSize' => 5 });
    ok( 5 == $Site->get('DefaultPageSize'), "set DefaultPageSize to numeric value" );

    $Site->set({ 'DefaultPageSize' => undef });
    ok( !defined $Site->get('DefaultPageSize'), "set DefaultPageSize to undef" );

    $Site->set({ 'DefaultPageSize' => $OrgDefaultPageSize });
    ok( $OrgDefaultPageSize == $Site->get('DefaultPageSize'), "set DefaultPageSize to default" );

    ok( Cleanup(), 'Cleanup' );
}

sub Cleanup {
    if( ExistsObjectByPath( $ROOTPATH ) ) {
        LoadObjectByPath( $ROOTPATH )->delete;
    }

    my $LocaleClass = LoadClassByAlias('Locale');
    if( $LocaleClass->existsObject('xx_YY') ) {
        $LocaleClass->loadObject('xx_YY')->delete;
    }

    return 1;
}

sub Init {
    Cleanup();

    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
