use strict;
use Test::More;
use DE_EPAGES::Core::API::File qw( GetFileContent DeleteDirectory);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Dictionary::API::Template qw ( ProcessFileTemplate );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath LoadRootObject LoadClassByAlias );
use DE_EPAGES::Object::API::TLE::ObjectHandler;

my $ATTRIBUTE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::DefaultAttribute';
my $ROOTPATH = '/Test';

our @Files;
our $TEST_ONE = undef;

BEGIN {
    @Files = glob 'TLE/*_t.txt';
    plan tests => (defined $TEST_ONE ? 1 : scalar @Files) + 2;
};

sub Main {
    ConnectTestSite();

    # create test classes
    ok( Init(), "initialization" );

    my $ParentObject = LoadObjectByPath( $ROOTPATH );
    my $Children = $ParentObject->children;
    my $Vars = { 'FirstLoop' => [map { $_->tleHash } @$Children] };

    my $TLEProcessor = new DE_EPAGES::TLE::API::BaseProcessor();
    my $ObjectHandler = DE_EPAGES::Object::API::TLE::ObjectHandler->new;
    $ObjectHandler->register($TLEProcessor);

    foreach my $File (@Files) {
        die "invalid file name $File" unless $File =~ /^(.+)_t\.txt$/;
        my $BaseFileName = $1;
        my $ReplacedContent = ProcessFileTemplate($BaseFileName.'_t.txt', 'en', $Vars, $TLEProcessor);
        my $CompareContent = ${GetFileContent($BaseFileName.'_e.txt')};
        $CompareContent =~ /^(.*)\n/;
        my $FirstLine = $1;
        ok($ReplacedContent eq $CompareContent, $FirstLine);
    }

    ok( Cleanup(), 'Cleanup' );
    DisconnectSite();
}

sub Cleanup {
    if( ExistsObjectByPath( $ROOTPATH ) ) {
        LoadObjectByPath( $ROOTPATH )->delete;
    }
    if( ExistsObjectByPath( '/Classes/C1' ) ) {
        LoadObjectByPath( '/Classes/C1' )->delete;
    }
    DeleteDirectory('ctmpl', {recursive=>1});
    return 1;
}

sub Init {
    Cleanup();

    local $SIG{__DIE__}  = \&DE_EPAGES::Core::API::Error::DieErrorHandler;
    local $SIG{__WARN__} = \&DE_EPAGES::Core::API::Error::DieErrorHandler;

    my $ParentObject = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    my $ClassObject = LoadClassByAlias( 'Object' );
    my $Class1 = $ClassObject->insertSubClass({ Alias => 'C1' });
    $Class1->insertAttribute({ Alias => 'StringVar', Type => 'String', Package => $ATTRIBUTE_PACKAGE });
    $Class1->insertAttribute({ Alias => 'IntegerVar', Type => 'Integer', Package => $ATTRIBUTE_PACKAGE });

    # add 10 objects
    for (my $i = 0; $i < 10; $i++) {
        my $Object = $ParentObject->insertChild( 'C1', {
            Alias => "NewObject$i",
            StringVar => "ab" . chr(ord("c") + $i),
            IntegerVar => $i
        } );
    }
    return 1;
}

RunScript(Sub=>\&Main, Debug=>1);
