use Test::Simple tests => 15;
use strict;
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadObject LoadRootObject LoadObjectByPath ExistsObjectByPath ExistsObject);
use DE_EPAGES::Order::API::Shop qw ( GetNextOrderNo );
use DE_EPAGES::Order::API::OrderDocument qw ( CreateOrderDocument );



my $ROOTPATH = '/Test';

sub Main {
    Init();

    my $System = LoadRootObject();
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShop1');
    ok( $Shop->id, "create shop " . $Shop->id );

    my $CustomerNo = GetNextCustomerNo( $Shop );
    ok( length $CustomerNo >= 4, "customer no. starts with 1000");

    my $CustomerFolder = GetCustomerFolder( $Shop );
    ok( !$CustomerFolder->existsChild( $CustomerNo ), "customer no. does not exists yet");

    my $Customer = $Shop->get('CustomerClass')->insertObject({
        Parent => $CustomerFolder,
        Alias => $CustomerNo,
    });
    ok( $Customer->id, "insert new customer" );
    ok( $Customer->alias eq $CustomerNo, "Customer.Alias" );

    my $OrderNo = GetNextOrderNo( $Shop );
    ok( length $OrderNo >= 4, "order no. starts with 1000");

    my $Order = $Customer->folder('Orders')->insertChild( 'CustomerOrder', {
        'Alias'     => $OrderNo,
        'Shop'      => $Shop,
        'Customer'  => $Customer,
    } );
    ok( $Order->id, "insert new order" );
    ok( $Order->alias eq $OrderNo, "Order.Alias" );
    ok( !defined $Order->get('LastStatus'), "new order has no status" );

    # add a line item container
    my $Container = $Order->insertChild('LineItemContainer', {
        'Alias'             => 'LineItemContainer',
        'LanguageID'        => $Shop->get('LanguageID'),
        'LocaleID'          => $Shop->get('LocaleID'),
        'CurrencyID'        => $Shop->get('CurrencyID'),
        'TaxArea'           => $Shop->get('TaxMatrix')->defaultTaxArea,
        'TaxModel'          => $Shop->get('TaxModel'),
    });
    $Order->set({'LineItemContainer' => $Container});

    # allow features for order documents
    $Shop->featureMaxValue( LoadObjectByPath('/Features/OrderDocument'), 5 );

    # Creation date from form
    my $DeliveryDate =  GetCurrentDateTime();
    my $IssueDate =     GetCurrentDateTime();

    CreateOrderDocument (
        $Order,
        'Invoice',
        'Invoices',
        {
            'DeliveryDate' => $DeliveryDate,
            'IssueDate' => $IssueDate
        }
    );
    my $Invoices = LoadObject($Order->findChild('Invoices'));
    my $NewInvoice = $Invoices->children->[0];
    ok(defined $NewInvoice, "new invoice successfully created with alias:".$NewInvoice->alias);

    CreateOrderDocument (
        $Order,
        'PackingSlip',
        'PackingSlips',
        {
            'DeliveryDate' => $DeliveryDate,
            'IssueDate' => $IssueDate
        }
    );
    my $PackingSlips = LoadObject($Order->findChild('PackingSlips'));
    my $NewPackingSlip = $Invoices->children->[0];
    ok(defined $NewPackingSlip, "new packing slip successfully created with alias:".$NewPackingSlip->alias);

    # Test set status
    $Order->set({'ReadyForShippingOn' => GetCurrentDateTime()});
    ok(defined $Order->get('ReadyForShippingOn'), "Status ReadyForShippingOn updated");

    # test LastStatus
    my $Now = GetCurrentDateTime();
    $Order->set({'ViewedOn' => $Now->clone->add( days => 1 )});
    ok( $Order->get('LastStatus')->alias eq 'ViewedOn', "Set ViewdOn" );

    $Order->set({'CancelledOn' => $Now->clone->add( days => 2 )});
    ok( $Order->get('LastStatus')->alias eq 'CancelledOn', "Set CancelledOn" );

    my $OrderID = $Order->id;
    $Customer->delete;
    ok( !ExistsObject($OrderID), "deleting the customer also deletes all orders" );

    Cleanup();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    my $Shop = $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );
    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
