use Test::More tests => 306;
use strict;
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObjectByPath ExistsObjectByPath );
use DE_EPAGES::Order::API::Shop qw ( GetNextOrderNo );
use DE_EPAGES::Order::API::OrderDocument qw ( CreateOrderDocument );
use Time::HiRes qw( gettimeofday tv_interval );



my $ROOTPATH = '/Test';

sub Main {
    Init();

    my $System = LoadRootObject();
    my $Shop = LoadObjectByPath($ROOTPATH.'/TestShop1');
    ok( $Shop->id, "create shop " . $Shop->id );

    my $CustomerNo = GetNextCustomerNo( $Shop );
    ok( length $CustomerNo >= 4, "customer no. starts with 1000");

    my $CustomerFolder = GetCustomerFolder( $Shop );
    ok( !$CustomerFolder->existsChild( $CustomerNo ), "customer no. does not exists yet");

    my $Customer = $Shop->get('CustomerClass')->insertObject({
        Parent => $CustomerFolder,
        Alias => $CustomerNo,
    });
    ok( $Customer->id, "insert new customer" );
    ok( $Customer->alias eq $CustomerNo, "Customer.Alias" );

    # allow features for order documents
    $Shop->featureMaxValue( LoadObjectByPath('/Features/OrderDocument'), 5 );

    my $OrderNo;
    my $Order;
    my $Container;
    my @Orders;

    my $i;
    for ($i= 0; $i < 100; $i++) {
      $OrderNo= GetNextOrderNo( $Shop );
      $Order= $Customer->folder('Orders')->insertChild( 'CustomerOrder', {
	  'Alias'     => $OrderNo,
	  'Shop'      => $Shop,
	  'Customer'  => $Customer,
      } );
      ok( $Order->id, "insert new order" );
      ok( $Order->alias eq $OrderNo, "Order.Alias" );
      ok( !defined $Order->get('LastStatus'), "new order has no status" );

      # add a line item container
      $Container = $Order->insertChild('LineItemContainer', {
	  'Alias'             => 'LineItemContainer',
	  'LanguageID'        => $Shop->get('LanguageID'),
	  'LocaleID'          => $Shop->get('LocaleID'),
	  'CurrencyID'        => $Shop->get('CurrencyID'),
	  'TaxArea'           => $Shop->get('TaxMatrix')->defaultTaxArea,
	  'TaxModel'          => $Shop->get('TaxModel'),
      });
      $Order->set({'LineItemContainer' => $Container});
      push(@Orders, $Order);
    }

    # Creation date from form
    my $DeliveryDate= GetCurrentDateTime();
    my $IssueDate=    GetCurrentDateTime();

    # Create an invoice and packing slip for each order.
    my $t0= [gettimeofday];
    foreach $Order (@Orders) {
      CreateOrderDocument (
	  $Order,
	  'Invoice',
	  'Invoices',
	  {
	      'DeliveryDate' => $DeliveryDate,
	      'IssueDate' => $IssueDate
	  }
      );

      CreateOrderDocument (
	  $Order,
	  'PackingSlip',
	  'PackingSlips',
	  {
	      'DeliveryDate' => $DeliveryDate,
	      'IssueDate' => $IssueDate
	  }
      );
    }
    my $t1= [gettimeofday];
    my $elapsed = tv_interval ($t0, $t1);

    cmp_ok( $elapsed, '<', 10, sprintf("Creating 100 invoices and packing slips takes less than 10 seconds. This run took: %0.2f seconds.", $elapsed) );

    Cleanup();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' });
    my $Shop = $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );
    return 1;
}

RunScript( Message => 'long', Sub=> sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});
