﻿use strict;
use Test::More;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath);
use DE_EPAGES::WebInterface::API::FakeRequest;

#========================================================================================
# §function     TimeThis
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       $Time = TimeThis( $Code );
#----------------------------------------------------------------------------------------
# §description  Executes a code block and returns the execution time.
#----------------------------------------------------------------------------------------
# §input        $Code | code to evaluate | code reference
# §return       $Time | execution time in seconds | float
#========================================================================================
sub TimeThis {
    my ($Code) = @_;

    my $time0 = time();
    $Code->();
    return time() - $time0;
}

#========================================================================================
# §function     Main
# §state        private
#========================================================================================
sub Main {

    my $NumTests = 21;
    plan tests => $NumTests;

    my $ObjectPath = "/Shops/DemoShop";
    my $DemoShopURL;
    my $ProductID;
    my $ProductURL;
    my $IsPageCacheUsed;
    my $LocaleID;
    RunOnStore(
        Store      => GetTestSite(),
        Sub        => sub {
            if( ExistsObjectByPath( $ObjectPath ) ) {
                my $Shop = LoadObjectByPath( $ObjectPath );
                $DemoShopURL = $Shop->get('WebUrl');
                $IsPageCacheUsed = $Shop->get('IsPageCacheUsed');
                $LocaleID = $Shop->get('LocaleID');

                my $Product = $Shop->get('ProductFolder')->child('ho_40407')->child('SubProducts')->child('ho_4040705005');
                $ProductID = $Product->id;
                $ProductURL = $Product->get('WebUrl');
            }
        }
    );

    SKIP: {
        skip "no DemoShop installed", $NumTests unless defined $DemoShopURL;

        my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
        my $GetURL = sub {
            my ($URL) = @_;

            my $Response = $FakeRequest->get( $URL );
            if( !$Response->is_success ) {
                die $Response->status_line;
            }
            else {
                return $Response->content;
            }
        };

        ok( !$IsPageCacheUsed, "Page Cache is disabled" );
        is( $LocaleID, 'en_GB', "default Locale is en_GB" );

        my $AddToBasketURL = $ProductURL . '&ChangeAction=AddToBasket&Quantity=2&ViewObjectID='.$ProductID;

        my $PreloadTime = TimeThis( sub {
            like( $GetURL->( $DemoShopURL ), qr!<title>Milestones - Equipped to meet your goals</title>!, "preload start page" );
            like( $GetURL->( $ProductURL ), qr!<title>Jack Wolfskin Blizzard Jacket - Milestones</title>!, "preload product page" );

            my $AddToBasketContent = $GetURL->( $AddToBasketURL );
            like( $AddToBasketContent, qr!Total amount!, "preload add to basket" );

            $AddToBasketContent =~ m!<a class="BasketLink" href="\?ObjectID=(\d+)">! or die "BasketID not found";
            my $BasketID = $1;

            $AddToBasketContent =~ m!<base href="(.*?)">! or die "<base> tag not found";
            my $BaseURL = $1;

            my $BasketURL = $BaseURL.'?ObjectID='.$BasketID;
            my $BasketContent = $GetURL->($BasketURL );
            like( $BasketContent, qr/Order and delivery comments/, "preload basket page" );

            my $LoginURL = $BaseURL.'?ObjectID='.$BasketID."&ChangeAction=SaveLoginForm&Login=mmustermann&Password=123456";
            my $LoginContent = $GetURL->( $LoginURL );
            like( $LoginContent, qr/Max Mustermann/, "preload login" );

            $LoginContent =~ m!<base href="(.*?)">! or die "<base> tag not found";
            $BaseURL = $1;

            my $OrderURL = $BaseURL.'?ObjectID='.$BasketID."&ChangeAction=TestTermsAndConditions&tac=1&ChangeAction=DoOrder";
            like( $GetURL->( $OrderURL ), qr/Confirmation/, "preload order" );
        } );
        cmp_ok( $PreloadTime, '<', 20, sprintf("preload: %0.2f s", $PreloadTime) );

#DB::enable_profile();
        my $StartPageTime = TimeThis( sub {
            like( $GetURL->( $DemoShopURL ), qr!<title>Milestones - Equipped to meet your goals</title>!, "start page" );
        } );
        cmp_ok( $StartPageTime, '<', 0.9, sprintf("start page: %0.2f s", $StartPageTime) );
#DB::disable_profile();

        my $ProductPageTime = TimeThis( sub {
            like( $GetURL->( $ProductURL ), qr!<title>Jack Wolfskin Blizzard Jacket - Milestones</title>!, "preload product page" );
        } );
        cmp_ok( $ProductPageTime, '<', 1, sprintf("product page: %0.2f s", $ProductPageTime) );

        # add to basket
        my $BaseURL;
        my $BasketID;
        my $AddToBasketTime = TimeThis( sub {
            my $AddToBasketContent = $GetURL->( $AddToBasketURL );
            like( $AddToBasketContent, qr!Total amount!, "add to basket" );

            $AddToBasketContent =~ m!<a class="BasketLink" href="\?ObjectID=(\d+)">! or die "BasketID not found";
            $BasketID = $1;

            $AddToBasketContent =~ m!<base href="(.*?)">! or die "<base> tag not found";
            $BaseURL = $1;
        } );
        cmp_ok( $AddToBasketTime, '<', 2, sprintf("add to basket: %0.2f s", $AddToBasketTime) );

        my $BasketURL = $BaseURL.'?ObjectID='.$BasketID;
        my $BasketTime = TimeThis( sub {
            my $BasketContent = $GetURL->($BasketURL );
            like( $BasketContent, qr/Order and delivery comments/, "basket page" );
        });
        cmp_ok( $BasketTime, '<', 1, sprintf("basket: %0.2f s", $BasketTime) );

        my $LoginTime = TimeThis( sub {
            my $LoginURL = $BaseURL.'?ObjectID='.$BasketID."&ChangeAction=SaveLoginForm&Login=mmustermann&Password=123456";
            my $LoginContent = $GetURL->( $LoginURL );
            like( $LoginContent, qr/Max Mustermann/, "login" );

            $LoginContent =~ m!<base href="(.*?)">! or die "<base> tag not found";
            $BaseURL = $1;
        });
        cmp_ok( $LoginTime, '<', 2, sprintf("login: %0.2f s", $LoginTime) );

        my $OrderTime = TimeThis( sub {
            my $OrderURL = $BaseURL.'?ObjectID='.$BasketID."&ChangeAction=TestTermsAndConditions&tac=1&ChangeAction=DoOrder";
            like( $GetURL->( $OrderURL ), qr/Confirmation/, "order" );
        });
        cmp_ok( $OrderTime, '<', 2, sprintf("order: %0.2f s", $OrderTime) );
    };

    return;
}

RunScript( Sub => \&Main, Debug => 1 );
