use strict;
use Test::More tests => 32;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

my $ROOTPATH = '/Test';

sub TestDownload {

    my $Product = LoadObjectByPath('/Test/TestShop/Products/DownloadProduct');
    ok( $Product->get('IsDownloadProduct'), "Product.IsDownloadProduct" );
    ok( !$Product->get('IsAppointment'), "not Product.IsAppointment" );

    my $aDownloads = $Product->get('DownloadProductMaps');
    is( scalar @$aDownloads, 1, "Product.DownloadProductMaps" );

    my $aDirectDownloads = $Product->get('DirectDownloadProductMaps');
    is( scalar @$aDownloads, 1, "Product.DirectDownloadProductMaps" );

    my $ProductEN = $Product->child('SubProducts')->child('DownloadProduct-en');
    $aDownloads = $ProductEN->get('DownloadProductMaps');
    is( scalar @$aDownloads, 2, "SubProducts.DownloadProductMaps" );
    is( $aDownloads->[0]->get('TargetUrl'), 'Product.exe', "files of master product before sub product" );
    is( $aDownloads->[1]->get('TargetUrl'), 'Product-en.dll', "files of sub product after master product" );

    $aDirectDownloads = $ProductEN->get('DirectDownloadProductMaps');
    is( scalar @$aDirectDownloads, 1, "SubProduct.DirectDownloadProductMaps" );

    my $ProductDE = $Product->child('SubProducts')->child('DownloadProduct-de');
    $aDownloads = $ProductDE->get('DownloadProductMaps');
    is( scalar @$aDownloads, 4, "DownloadProductMaps of DownloadProduct-de" );
    my $DownloadLocal = $aDownloads->[0];
    is( $DownloadLocal->get('IsExternal'), 0, "DownloadProductMap.IsExternal" );
    is( $DownloadLocal->get('FileName'), 'Product.exe', 'DownloadProductMap.FileName for local file' );

    my $DownloadExtern = $aDownloads->[3];
    is( $DownloadExtern->get('IsExternal'), 1, "DownloadProductMap.IsExternal" );
    is( $DownloadExtern->get('FileName'), 'epages7.exe', 'DownloadProductMap.FileName for external file' );
}

sub TestAppointment {

    my $Product = LoadObjectByPath('/Test/TestShop/Appointments/Table4Person');
    ok( !$Product->get('IsDownloadProduct'), "not Product.IsDownloadProduct" );
    ok( $Product->get('IsAppointment'), "Product.IsAppointment" );

    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new('locale'=>'de_DE');
    my $StartDate = $Formatter->parse_datetime('20.02.2008 16:20:00','datetime', 'medium');
    my $EndDate1 = $Formatter->parse_datetime('20.02.2009 16:13:30','datetime', 'medium');
    my $EndDate2 = $Formatter->parse_datetime('20.02.2009 16:23:30','datetime', 'medium');
    my $EndDate3 = $Formatter->parse_datetime('20.02.2010 16:23:30','datetime', 'medium');
    my $EndDate4 = $Formatter->parse_datetime('20.02.2008 16:50:00','datetime', 'medium');
    my $EndDate5 = $Formatter->parse_datetime('20.02.2008 17:20:00','datetime', 'medium');
    my $EndDate6 = $Formatter->parse_datetime('20.02.2008 18:20:00','datetime', 'medium');

    is ($Product->timeFactor($StartDate,$EndDate1), 4391.9458333333, '2 hours without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 4392.0291666667, '2 hours with overlap');
    is ($Product->timeFactor($StartDate,$EndDate4), 0.25, '1/2 hour');
    is ($Product->timeFactor($StartDate,$EndDate5), 0.5, '1 hour');
    is ($Product->timeFactor($StartDate,$EndDate6), 1, '2 hours');

    $Product->set({DurationUnit=>LoadObjectByPath("/Units/second/month"), DurationAmount=>1});

    is ($Product->timeFactor($StartDate,$EndDate1), 11.5, 'month without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 12.5, 'month with overlap');

    $Product->set({DurationUnit=>LoadObjectByPath("/Units/second/month"), DurationAmount=>2});
    is ($Product->timeFactor($StartDate,$EndDate1), 5.75, '2 month without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 6.25, '2 month with overlap');

    $Product->set({DurationUnit=>LoadObjectByPath("/Units/second/year"), DurationAmount=>1});
    is ($Product->timeFactor($StartDate,$EndDate1), 0.9583333333, 'year without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 1.0416666667, 'year with overlap');
    is ($Product->timeFactor($StartDate,$EndDate3), 2.0416666667, 'year with more overlap');

    $Product->set({DurationUnit=>LoadObjectByPath("/Units/second/year"), DurationAmount=>2});
    is ($Product->timeFactor($StartDate,$EndDate1), 0.4791666667, '2 year without overlap');
    is ($Product->timeFactor($StartDate,$EndDate2), 0.5208333333, '2 years with overlap');
    is ($Product->timeFactor($StartDate,$EndDate3), 1.0208333333, '2 years with more overlap');
}

sub Cleanup {
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    Cleanup();
    LoadRootObject()->insertChild('Object', {'Alias'=>'Test'});
    ImportObjects('import.xml', $ROOTPATH);
    return 1;
}

RunScript(Debug=>1, Sub=> sub {
    ConnectTestSite();
    ok( Init(), 'initialization' );

    TestDownload();
    TestAppointment();

    ok( Cleanup(), 'cleanup');
    DisconnectSite();
});
