use Test::More tests => 39;
use strict;
use DateTime;
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode );

local $ENV{'EPAGES_TESTSITE'} = 'Site';

sub Main {
    Cleanup();

    my $System = LoadRootObject();

    # these features are not counted (IsFree=1)
    my $Feature1 = $System->folder('FeatureRefs')->child( 'PaymentMethods' );
    my $Feature2 = $System->folder('FeatureRefs')->child( 'ShippingMethods' );

    # insert a store
    my $Store = $System->folder('Stores')->child('Store');

    # get a provider
    my $Provider = $System->folder('Providers')->child('Distributor');

    my $EN = GetPKeyLanguageByCode( 'en' );
    local $DE_EPAGES::Object::API::Language::LANGUAGEID = $EN;

    # new shop type
    my $ShopType1 = $Provider->insertShopType( {
        Alias => 'ST1',
        Name => 'ShopType1',
        ShopImportFile => 'Empty.xml',
    } );
    $ShopType1->featureMaxValue( $Feature2, 50 );
    ok( 50 == $ShopType1->featureMaxValue( $Feature2 ), "check ShopType.featureMaxValue" );

    # new feature pack
    my $FeaturePack1 = $Provider->insertFeaturePack( {
        Alias => 'FP1',
        Name => 'FeaturePack1',
        AllowMax => 2
    } );
    $FeaturePack1->featureMaxValue( $Feature1, 1 );
    $FeaturePack1->featureMaxValue( $Feature2, 15 );

    # create a new shop reference
    ok( !$Provider->existsShop( 'Sh1' ), "before insertShop" );
    my $ShopRef = $Provider->insertShop( {
        Alias => 'Sh1',
        ShopType => $ShopType1,
        Store => $System->folder('Stores')->child('Store'),
        IsClosed => 0,
    } );

    ok( $Provider->existsShop( 'Sh1' ), "after insertShop" );
    ok( $ShopRef->id == $Provider->shop( 'Sh1' )->id, "Provider.shop" );
    ok( $ShopRef->get('Provider')->id == $Provider->id, "Shop.Provider" );
    ok( !$ShopRef->get('IsClosed'), "Shop.IsClosed" );
    ok( !defined $ShopRef->get('ClosedOn'), "Shop.ClosedOn" );
    ok( !$ShopRef->get('IsDeleted'), "Shop.IsDeleted" );
    ok( !defined $ShopRef->get('DeletedOn'), "Shop.DeletedOn" );
    ok( !$ShopRef->get('IsTrialShop'), "Shop.IsTrialShop" );
    ok( !defined $ShopRef->get('TrialToLiveOn'), "Shop.TrialToLiveOn" );
    ok( !defined $ShopRef->get('MarkedForDelOn'), "Shop.MarkedForDelOn" );
    ok( $ShopRef->get('ShopType')->id == $ShopType1->id, "Shop.ShopType" );

    # change IsClosed
    my $Now = GetCurrentDBHandle()->currentDateTime;
    $ShopRef->set({ IsClosed => 1 });
    ok( $ShopRef->get('IsClosed'), "set IsClosed" );
    cmp_ok( $ShopRef->get('ClosedOn')->ymd, 'eq', $Now->ymd, "IsClosed sets ClosedOn" );
    $ShopRef->set({ IsClosed => 0 });
    ok( !$ShopRef->get('IsClosed'), "reset IsClosed" );
    ok( !defined $ShopRef->get('ClosedOn'), "IsClosed resets ClosedOn" );

    # change MarkedForDelOn
    $ShopRef->set({ MarkedForDelOn => $Now });
    cmp_ok( $ShopRef->get('MarkedForDelOn')->ymd, 'eq', $Now->ymd, "set MarkedForDelOn" );
    cmp_ok( $ShopRef->get('ClosedOn')->ymd, 'eq', $Now->ymd, "MarkedForDelOn sets ClosedOn" );
    $ShopRef->set({ MarkedForDelOn => undef });
    ok( !defined $ShopRef->get('MarkedForDelOn'), "reset MarkedForDelOn" );
    cmp_ok( $ShopRef->get('ClosedOn')->ymd, 'eq', $Now->ymd, "MarkedForDelOn does not reset ClosedOn" );
    $ShopRef->set({ IsClosed => 0 });

    # change IsTrialShop
    $ShopRef->set({ IsTrialShop => 1 });
    ok( $ShopRef->get('IsTrialShop'), "set IsTrialShop" );
    $ShopRef->set({ IsTrialShop => 0 });
    ok( !$ShopRef->get('IsTrialShop'), "reset IsTrialShop" );
    ok( $Now->ymd eq $ShopRef->get('TrialToLiveOn')->ymd, "IsTrialShop sets TrialToLiveOn" );
    $ShopRef->set({ IsTrialShop => 1 });
    ok( $ShopRef->get('IsTrialShop'), "set IsTrialShop" );
    ok( !defined $ShopRef->get('TrialToLiveOn'), "IsTrialShop resets TrialToLiveOn" );

    # add feature packs
    ok( 1 == scalar @{ $ShopRef->featureCollections }, "shoptype is added as feature collection" );
    ok( $ShopType1->id == $ShopRef->featureCollections->[0]->get('FeatureCollection')->id, "shop type id" );

    my %FeatureMaxValues = map { ( $_->{FeatureRef}->id, $_->{MaxValue} ) } @{ $ShopRef->features };
    ok( 50 == $FeatureMaxValues{ $Feature2->id }, "feature max value of shop" );
    my $SFC1 = $ShopRef->addFeaturePack( $FeaturePack1 );
    my $SFC2 = $ShopRef->addFeaturePack( $FeaturePack1 );
    ok( $SFC1->get('ShopRef')->id == $ShopRef->id, "ShopFeatureCollection.shop" );
    ok( $SFC1->get('FeatureCollection')->id == $FeaturePack1->id, "ShopFeatureCollection.featureCollection" );
    ok( $SFC1->get('IsActive'), "ShopFeatureCollection.IsActive" );
    ok( 3 == scalar @{ $ShopRef->featureCollections }, "feature packs are added" );
    ok( 2 == scalar @{ $SFC1->get('Features') }, "feature count" );

    %FeatureMaxValues = map { ( $_->{FeatureRef}->id => $_->{MaxValue} ) } @{ $ShopRef->features };
    ok( 2 == $FeatureMaxValues{ $Feature1->id }, "feature value including feature packs 1" );
    ok( 80 == $FeatureMaxValues{ $Feature2->id }, "feature value including feature packs 2" );

    # remove feature packs
    ok( $FeatureMaxValues{ $Feature1->id } > 0, "shop has feature 1" );
    $SFC1->set({ 'IsActive'=>0 });
    $SFC2->set({ 'IsActive'=>0 });

    %FeatureMaxValues = map { ( $_->{FeatureRef}->id => $_->{MaxValue} ) } @{ $ShopRef->features };
    ok( 50 == $FeatureMaxValues{ $Feature2->id }, "feature value after remove feature pack" );
    ok( 0 == $FeatureMaxValues{ $Feature1->id }, "feature 1 has been removed" );
    Cleanup();
}

RunScript( Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => \&Main );
});

sub Cleanup {

    my $System = LoadRootObject();

    my $Provider = $System->folder('Providers')->child('Distributor');
    $Provider->shop('Sh1')->delete if $Provider->existsShop('Sh1');
    $Provider->shopType('ST1')->delete if $Provider->existsShopType('ST1');
    $Provider->featurePack('FP1')->delete if $Provider->existsFeaturePack('FP1');
}
