use Test::Simple tests => 4;
use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Database::API::Connection qw ( ConnectTestSite DisconnectSite );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Trigger::API::XML::HookImportHandler qw ( ImportHooks );
use DE_EPAGES::Trigger::API::XML::HookDeleteHandler qw ( DeleteHooks );
use DE_EPAGES::Trigger::API::Table::Hook qw ( InsertHook GetPKeyHooksByName DeleteHook );

our $CALLED = 0;

sub Main {
    local $| = 1;
    local $ENV{SCRIPT_NAME} = __FILE__;
    ConnectTestSite();
    Cleanup();
    ok ( Test_Import(), "import ok" );
    ok ( Test_Trigger(), "trigger ok" );
    ok ( Test_Export(), "export ok" );
    ok ( Test_ExportFalse(), "find wrong export" );
    Cleanup();
    DisconnectSite();
}

sub Test_Import {
    eval {
        ImportHooks('01_s.xml');
        ImportHooks('01_s.xml');
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}
sub Test_Trigger {
    eval {
        TriggerHook('TestHook', { 'TestParameter'=>'Param' });
        Error('NO_HOOK_NOTCALLED') unless $CALLED;
        $CALLED = 0;
        TriggerHook('TestHookPackage', { 'TestParameter'=>'Param' });
        Error('NO_HOOK_NOTCALLED') unless $CALLED;
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}

sub Test_Export {
    eval {
        ImportHooks('01_s.xml');
        my $Driver = TestDriver->new();
        $Driver->exportXML('01_e.xml',{ encoding => 'iso-8859-1' });
        my $compareResult = CompareFiles('01_s.xml', '01_e.xml', sub {
            local $_ = shift;
            s/'/"/g;
            s/delete="1"//g;
            s/\s//g;
            return $_;
        });
        Error('NOT_EQUAL') if $compareResult;
        unlink '01_e.xml';
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}

sub Test_ExportFalse {
    eval {
        ImportHooks('01_s.xml');
        my $HookID = InsertHook({'Name'=>'TestHook3'});
        my $Driver = TestDriver->new();
        $Driver->exportXML('01_e.xml',{ encoding => 'iso-8859-1' });
        my $compareResult = CompareFiles('01_s.xml', '01_e.xml', sub {
            local $_ = shift;
            s/'/"/g;
            s/delete="1"//g;
            s/\s//g;
            return $_;
        });
        Error('NOT_EQUAL') unless $compareResult;
        unlink '01_e.xml';
    };
    if (ExistsError()) {
        my $Error = GetError();
        print $Error->longMessage();
        return 0;
    }
    return 1;
}

sub TestHookProc {
    my ($args) = @_;
    Error('NO_HOOK_NAME') unless defined $args->{'HookName'}      and $args->{'HookName'} eq 'TestHook';
    Error('NO_HOOK_PARA') unless defined $args->{'TestParameter'} and $args->{'TestParameter'} eq 'Param';
    $CALLED = 1;
}

sub CompareFiles {
    my ($file1, $file2, $cExec) = @_;
    my $rContent1 = GetFileContent($file1);
    my $rContent2 = GetFileContent($file2);
    if (defined $cExec) {
        $$rContent1 = &$cExec($$rContent1);
        $$rContent2 = &$cExec($$rContent2);
    }
    return $$rContent1 cmp $$rContent2;
}

sub Cleanup {
    DeleteHooks('01_s.xml');
    my $IDs = GetPKeyHooksByName('TestHook3');
    DeleteHook($_) foreach @$IDs;
}

RunScript(
    'Sub' => \&Main
);

package HookT;
use strict;
use DE_EPAGES::Core::API::Error qw ( Error );

sub TestHookProc {
    my ($args) = @_;
    Error('NO_HOOK_NAME') unless defined $args->{'HookName'}      and $args->{'HookName'} eq 'TestHookPackage';
    Error('NO_HOOK_PARA') unless defined $args->{'TestParameter'} and $args->{'TestParameter'} eq 'Param';
    $CALLED = 1;
}

##no critic(ProhibitMultiplePackages)
package TestDriver;
##use critic
use base qw( DE_EPAGES::Trigger::API::XML::HookExportDriver );

use strict;
use DE_EPAGES::Trigger::API::Table::Hook qw( GetPKeyHooksByName );

sub AllHooks {
    my $self = shift;
    my $IDs = GetPKeyHooksByName('TestHook');
    $self->addElement( 'Hook', { 'HookID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyHooksByName('TestHookPackage');
    $self->addElement( 'Hook', { 'HookID' => $_ } ) foreach @$IDs;
    $IDs = GetPKeyHooksByName('TestHook3');
    $self->addElement( 'Hook', { 'HookID' => $_ } ) foreach @$IDs;
}

# get attributes for the document tag
sub epages {
    return {};
}

sub addHookHooks {
    my $self = shift;
    $self->addHook( 'Hook' => 'HookFunctionsByHook' );
}

sub parse {
    my $self = shift;

    $self->addHook( 'epages' => 'AllHooks' );
    $self->addHookHooks;

    $self->SUPER::start_document();
    $self->xml_decl({'Version'=>'1.0', 'Encoding'=>$self->encodeTo});
    $self->characters({'Data' => "\n"});
    $self->addElement( 'epages' );
    $self->SUPER::end_document();
}

1;

