use strict;
use Test::More tests => 17;
use DE_EPAGES::Core::API::Error qw( ExistsError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Net qw( SupportedIpVersions );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf );
use DE_EPAGES::WebInterface::API::Constants;
use DE_EPAGES::WebInterface::API::MessageCenter;

RunScript( Sub => sub {
    my $MessageCenter = DE_EPAGES::WebInterface::API::MessageCenter->new(
        AppServerAddress => '127.0.0.1',
        AppServerPort    => '10048',
        AppServerPoolName => 'DefaultPool'
    );
    $MessageCenter->connect;
    ok (1,"connect");
    $MessageCenter->sendInitialFreeRequest;
    ok (1,"initial");
    $MessageCenter->sendWorkingRequest;
    ok (1,"busy");
    $MessageCenter->sendMonitorRequest;
    ok (1,"monitor");
    $MessageCenter->sendFinishRequest;
    ok (1,"finish");

    $MessageCenter = DE_EPAGES::WebInterface::API::MessageCenter->new(
        AppServerAddress => '127.0.0.1',
        AppServerPort    => '10048',
        AppServerPoolName => 'DefaultPool'
    );
    $MessageCenter->connect;
    $MessageCenter->sendPoolReset;
    ok (1,"sendPoolReset");

    # transaction id 0 is not checked
    my $aSQL = [ "SELECT storename FROM v_lookup", "SELECT siteguid FROM v_lookup" ];
    $MessageCenter->sendPoolChanges( 0, $aSQL );
    ok( 1, "sendPoolChanges" );

    # use wrong transaction id -> reloads the pooldb from cache, not an error
    eval { $MessageCenter->sendPoolChanges( 99999999, $aSQL ) };
    ok( !ExistsError(), "sendPoolChanges wrong transaction id" );

    # invalid SQL causes an error
    my $aInvalidSQL = [ "SELECT storename FROM v_lookup", "SELECT x FROM y" ];
    eval { $MessageCenter->sendPoolChanges( 0, $aInvalidSQL ) };
    ok( ExistsError(), "sendPoolChanges error" );

    # must re-connect, because RequestRouter closes the connection after error
    $MessageCenter = DE_EPAGES::WebInterface::API::MessageCenter->new(
        AppServerAddress => '127.0.0.1',
        AppServerPort    => '10048',
        AppServerPoolName => 'DefaultPool'
    );
    $MessageCenter->connect;

    # must re-connect, because RequestRouter closes the connection after error
    $MessageCenter = DE_EPAGES::WebInterface::API::MessageCenter->new(
        AppServerAddress => '127.0.0.1',
        AppServerPort    => '10048',
        AppServerPoolName => 'DefaultPool'
    );
    $MessageCenter->connect;

    # emulate broadcast requests
    my $OutData = $MessageCenter->_request( MCREQUEST_POOLRESET, '1' );
    ok( "" eq $OutData, "emulate MCREQUEST_POOLRESET broadcast" );

    my $RequestLine = "0 1";
    my $Data = join '', map { s/\r?\n/ /g; $_ . POOLDB_SQL_DELIMITER; } @$aSQL;
    $OutData = $MessageCenter->_request( MCREQUEST_POOLCHANGES, $RequestLine, $Data );
    ok( "" eq $OutData, "emulate MCREQUEST_POOLCHANGES broadcast" );

    $Data = join '', map { s/\r?\n/ /g; $_ . POOLDB_SQL_DELIMITER; } @$aInvalidSQL;
    eval { $MessageCenter->_request( MCREQUEST_POOLCHANGES, $RequestLine, $Data ); };
    ok( ExistsError(), "emulate MCREQUEST_POOLCHANGES broadcast error" );

    my $SkipIPv6;
    $SkipIPv6 = "OS doesn't support IPv6" unless ElementOf(6, SupportedIpVersions());
    $SkipIPv6 = 'envvar EPAGES_ENABLE_IPv6 not set' unless $ENV{EPAGES_ENABLE_IPv6};

    if ( $SkipIPv6 ) {
      SKIP: { skip(": $SkipIPv6", 5); }
    } else {
        my $MessageCenter2 = DE_EPAGES::WebInterface::API::MessageCenter->new(
            AppServerAddress => '::1',
            AppServerPort    => '10048',
            AppServerPoolName => 'DefaultPool'
            );
        $MessageCenter2->connect;
        ok (1,"connect");
        $MessageCenter2->sendInitialFreeRequest;
        ok (1,"initial");
        $MessageCenter2->sendWorkingRequest;
        ok (1,"busy");
        $MessageCenter2->sendMonitorRequest;
        ok (1,"monitor");
        $MessageCenter2->sendFinishRequest;
        ok (1,"finish");
    }
});
